@@ -356,9 +356,9 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	gpsd_report(LOG_INF, "Garmin: Product ID: %d, SoftVer: %d.%02d\n",
 		    prod_id, maj_ver, min_ver);
 	gpsd_report(LOG_INF, "Garmin: Product Desc: %s\n", &buf[4]);
-	mask |= DEVICEID_IS;
-	gpsd_report(LOG_DATA, "Garmin: PRODUCT_DATA: subtype=%s mask=%s\n",
-		    session->subtype, gpsd_maskdump(mask));
+	mask |= DEVICEID_SET;
+	gpsd_report(LOG_DATA, "Garmin: PRODUCT_DATA: subtype=%s\n",
+		    session->subtype);
 	break;
     case GARMIN_PKTID_PVT_DATA:
 	gpsd_report(LOG_PROG, "Garmin: Appl, PVT Data Sz: %d\n", pkt_len);
@@ -478,15 +478,22 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	if (session->newdata.mode > MODE_NO_FIX) {
 	    /* data only valid with a fix */
 	    mask |=
-		TIME_IS | LATLON_IS | ALTITUDE_IS | STATUS_IS | MODE_IS |
-		SPEED_IS | TRACK_IS | CLIMB_IS | HERR_IS | VERR_IS | PERR_IS |
+		TIME_SET | LATLON_SET | ALTITUDE_SET | STATUS_SET | MODE_SET |
+		SPEED_SET | TRACK_SET | CLIMB_SET | HERR_SET | VERR_SET | PERR_IS |
 		CLEAR_IS | REPORT_IS;
+	    /*
+	     * Garmin documentation says we should wait until four good fixes
+	     * have been seen before trying to use the device for precision 
+	     * time service.
+	     */
+	    if (session->fixcnt > 3)
+		mask |= PPSTIME_IS;
 	}
 	gpsd_report(LOG_DATA,
 		    "Garmin: PVT_DATA: time=%.2f, lat=%.2f lon=%.2f "
 		    "speed=%.2f track=%.2f climb=%.2f "
 		    "epx=%.2f epy=%.2f epv=%.2f "
-		    "mode=%d status=%d mask=%s\n",
+		    "mode=%d status=%d\n",
 		    session->newdata.time,
 		    session->newdata.latitude,
 		    session->newdata.longitude,
@@ -497,7 +504,7 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 		    session->newdata.epy,
 		    session->newdata.epv,
 		    session->newdata.mode,
-		    session->gpsdata.status, gpsd_maskdump(mask));
+		    session->gpsdata.status);
 	break;
     case GARMIN_PKTID_RMD_DATA:
     case GARMIN_PKTID_RMD41_DATA:
@@ -561,11 +568,11 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 
 	}
 	session->gpsdata.skyview_time = NAN;
-	mask |= SATELLITE_IS | USED_IS;
+	mask |= SATELLITE_SET | USED_IS;
 	gpsd_report(LOG_DATA,
-		    "Garmin: SAT_DATA: visible=%d used=%d mask=%s\n",
+		    "Garmin: SAT_DATA: visible=%d used=%d\n",
 		    session->gpsdata.satellites_visible,
-		    session->gpsdata.satellites_used, gpsd_maskdump(mask));
+		    session->gpsdata.satellites_used);
 	break;
     case GARMIN_PKTID_PROTOCOL_ARRAY:
 	// this packet is never requested, it just comes, in some case
@@ -579,14 +586,12 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	break;
     default:
 	gpsd_report(LOG_WARN,
-		    "Garmin: Unknown packet id: %#02x, Sz: %#02x, pkt:%s\n",
-		    pkt_id, pkt_len, gpsd_hexdump_wrapper(buf,
-							  (size_t) pkt_len,
-							  LOG_WARN));
+		    "Garmin: Unknown packet id: %#02x, Sz: %#02x\n",
+		    pkt_id, pkt_len);
 	break;
     }
-    gpsd_report(LOG_IO, "Garmin: PrintSERPacket(, %#02x, %#02x, ) = %s\n",
-		pkt_id, pkt_len, gpsd_maskdump(mask));
+    gpsd_report(LOG_IO, "Garmin: PrintSERPacket(, %#02x, %#02x, )\n",
+		pkt_id, pkt_len);
     return mask;
 }
 
@@ -713,11 +718,6 @@ static void Build_Send_USB_Packet(struct gps_device_t *session,
     } else if (4 == length) {
 	set_int32(buffer + 12, data);
     }
-#if 0
-    gpsd_report(LOG_IO, "Garmin: SendPacket(), writing %d bytes: %s\n",
-		theBytesToWrite,
-		gpsd_hexdump_wrapper(thePacket, theBytesToWrite, LOG_IO));
-#endif
     (void)PrintUSBPacket(session, thePacket);
 
     theBytesReturned = gpsd_write(session, (const char *)thePacket,
@@ -735,7 +735,7 @@ static void Build_Send_USB_Packet(struct gps_device_t *session,
     // So here goes just in case
     if (0 == (theBytesToWrite % ASYNC_DATA_SIZE)) {
 	char *n = "";
-	theBytesReturned = gpsd_write(session, n, 0);
+	(void)gpsd_write(session, n, 0);
     }
 }
 #endif /* __linux__ || S_SPLINT_S */
@@ -785,14 +785,9 @@ static void Build_Send_SER_Packet(struct gps_device_t *session,
     }
     // Add DLE, ETX
     *buffer++ = (uint8_t) DLE;
-    *buffer++ = (uint8_t) ETX;
+    /* we used to say n++ here, but scan-build complains */
+    *buffer = (uint8_t) ETX;
 
-#if 1
-    gpsd_report(LOG_IO, "Garmin: SendPacket(), writing %zd bytes: %s\n",
-		theBytesToWrite,
-		gpsd_hexdump_wrapper(thePacket, (size_t) theBytesToWrite,
-				     LOG_IO));
-#endif
     (void)PrintSERPacket(session,
 			 (unsigned char)buffer0[1],
 			 (int)buffer0[2], (unsigned char *)(buffer0 + 3));
@@ -804,7 +799,7 @@ static void Build_Send_SER_Packet(struct gps_device_t *session,
 
 }
 
-#if defined(HAVE_LIBUSB) || defined(S_SPLINT_S)
+#if defined(HAVE_LIBUSB)
 /*
  * is_usb_device() - is a specified device USB matching given vendor/product?
  *
@@ -864,7 +859,7 @@ static bool is_usb_device(const char *path UNUSED, int vendor, int product)
 }
 
 /*@-compdef -usedef -nullpass@*/
-#endif /* HAVE_LIBUSB || S_SPLINT_S */
+#endif /* HAVE_LIBUSB */
 
 /*
  * garmin_usb_detect() - detect a Garmin USB device connected to session fd.
@@ -909,7 +904,7 @@ static bool garmin_usb_detect(struct gps_device_t *session UNUSED)
 #else
 	return false;
 #endif /* HAVE_LIBUSB */
-
+	/*@-unreachable@*/
 	if (!gpsd_set_raw(session)) {
 	    gpsd_report(LOG_ERROR,
 			"Garmin: garmin_usb_detect: error changing port attributes: %s\n",
@@ -1091,7 +1086,8 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
 		    n + i);
 	return 0;
     }
-    c = buf[n + i++];
+    /* we used to say n++ here, but scan-build complains */
+    c = buf[n + i];
     if ('\x03' != c) {
 	Send_NAK();
 	gpsd_report(LOG_RAW + 1, "Garmin: Final ETX not ETX\n");
@@ -1115,14 +1111,13 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     (void)usleep(300);
     Send_ACK();
     /*@ +usedef +compdef @*/
-    gpsd_report(LOG_IO, "Garmin: garmin_ser_parse( ) = %s\n",
-		gpsd_maskdump(mask));
+    gpsd_report(LOG_IO, "Garmin: garmin_ser_parse( )\n");
     return mask;
 }
 
 /*@ -charint @*/
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static void settle(void)
 {
     struct timespec delay, rem;
@@ -1163,9 +1158,9 @@ static void garmin_switcher(struct gps_device_t *session, int mode)
 	settle();		// wait 333mS, essential!
     }
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 static ssize_t garmin_control_send(struct gps_device_t *session,
 				   char *buf, size_t buflen)
 /* not used by the daemon, it's for gpsctl and friends */
@@ -1176,7 +1171,7 @@ static ssize_t garmin_control_send(struct gps_device_t *session,
     return gpsd_write(session, session->msgbuf, session->msgbuflen);
     /*@ +mayaliasunique @*/
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
 #ifdef NTPSHM_ENABLE
 static double garmin_ntp_offset(struct gps_device_t *session)
@@ -1258,12 +1253,6 @@ static int GetPacket(struct gps_device_t *session)
 	    continue;
 	}
 	gpsd_report(LOG_RAW, "Garmin: got %d bytes\n", theBytesReturned);
-#if 1
-	gpsd_report(LOG_IO, "Garmin: getPacket(), got %d bytes: %s\n",
-		    theBytesReturned,
-		    gpsd_hexdump_wrapper(thePacket, theBytesReturned,
-					 LOG_IO));
-#endif
 
 	session->driver.garmin.BufferLen += theBytesReturned;
 	if (256 <= session->driver.garmin.BufferLen) {
@@ -1340,15 +1329,15 @@ const struct gps_type_t garmin_usb_binary_old =
     .parse_packet   = garmin_usb_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
     .event_hook     = garmin_event_hook,/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = garmin_control_send,	/* send raw bytes */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = garmin_ntp_offset,
 #endif /* NTPSHM_ ENABLE */
@@ -1369,15 +1358,15 @@ const struct gps_type_t garmin_usb_binary =
     .parse_packet   = garmin_ser_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
     .event_hook     = garmin_event_hook,/* lifetime ebent handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,	        /* Garmin USB Binary has no NMEA */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = garmin_control_send,	/* send raw bytes */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = garmin_ntp_offset,
 #endif /* NTPSHM_ ENABLE */
@@ -1397,15 +1386,15 @@ const struct gps_type_t garmin_ser_binary =
     .parse_packet   = garmin_ser_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
     .event_hook     = NULL,	        /* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = garmin_switcher,	/* how to change modes */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = garmin_control_send,	/* send raw bytes */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = garmin_ntp_offset,
 #endif /* NTPSHM_ ENABLE */
