@@ -68,9 +68,7 @@ static int geostar_write(struct gps_device_t *session,
 
     session->msgbuflen = len * 4;
 
-    gpsd_report(LOG_IO, "Sent GeoStar packet id 0x%x: %s\n", id,
-		gpsd_hexdump_wrapper(session->msgbuf, session->msgbuflen, LOG_IO));
-
+    gpsd_report(LOG_IO, "Sent GeoStar packet id 0x%x\n", id);
     if (gpsd_write(session, session->msgbuf, session->msgbuflen) !=
 	(ssize_t) session->msgbuflen)
 	return -1;
@@ -204,14 +202,14 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 
 	if (ul1 != 0) {
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    mask |= STATUS_IS;
+	    mask |= STATUS_SET;
 	} else {
 	    if (session->gpsdata.status < STATUS_FIX) {
 		session->gpsdata.status = STATUS_FIX;
-		mask |= STATUS_IS;
+		mask |= STATUS_SET;
 	    }
 	}
-	mask |= TIME_IS | LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | DOP_IS | USED_IS | REPORT_IS;
+	mask |= TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | DOP_SET | USED_IS | REPORT_IS;
 
 	gpsd_report(LOG_INF, "Geographic coordinates %f %g %g %g %g %g\n",
 		d1,
@@ -247,7 +245,7 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 	    session->newdata.mode = MODE_NO_FIX;
 	}
 
-	mask |= MODE_IS | STATUS_IS;
+	mask |= MODE_SET | STATUS_SET;
 	break;
     case 0x22:
 	ul1 = getleu32(buf, OFFSET(1));
@@ -270,7 +268,7 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 	    }
 	}
 	session->gpsdata.skyview_time = NAN;
-	mask |= SATELLITE_IS | USED_IS;
+	mask |= SATELLITE_SET | USED_IS;
 	break;
     case 0x3e:
 	ul1 = getleu32(buf, OFFSET(1));
@@ -414,7 +412,7 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 	/*@ +formattype @*/
 	gpsd_report(LOG_INF, "Response to Request FW version command: %s\n",
 		    session->subtype);
-	mask |= DEVICEID_IS;
+	mask |= DEVICEID_SET;
 	break;
     case 0xc2:
 	gpsd_report(LOG_INF, "Response to Restart receiver command\n");
@@ -451,7 +449,7 @@ static gps_mask_t geostar_parse_input(struct gps_device_t *session)
 	return 0;
 }
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 static ssize_t geostar_control_send(struct gps_device_t *session,
 				 char *buf, size_t buflen)
 /* not used by the daemon, it's for gpsctl and friends */
@@ -460,7 +458,7 @@ static ssize_t geostar_control_send(struct gps_device_t *session,
 				(unsigned int)buf[0],
 				(unsigned char *)buf + 1, (buflen - 1)/4);
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
 static void geostar_event_hook(struct gps_device_t *session, event_t event)
 {
@@ -509,7 +507,7 @@ static void geostar_event_hook(struct gps_device_t *session, event_t event)
     /*@+shiftimplementation -ignoresigns@*/
 }
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static bool geostar_speed_switch(struct gps_device_t *session,
 			      speed_t speed, char parity, int stopbits)
 {
@@ -559,7 +557,7 @@ static void geostar_mode(struct gps_device_t *session, int mode)
     }
     /*@+shiftimplementation@*/
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 #ifdef NTPSHM_ENABLE
 static double geostar_ntp_offset(struct gps_device_t *session UNUSED)
@@ -582,15 +580,15 @@ const struct gps_type_t geostar_binary =
     .parse_packet   = geostar_parse_input,	/* parse message packets */
     .rtcm_writer    = NULL,		/* doesn't accept DGPS corrections */
     .event_hook     = geostar_event_hook,	/* fire on various lifetime events */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = geostar_speed_switch,/* change baud rate */
     .mode_switcher  = geostar_mode,	/* there is a mode switcher */
     .rate_switcher  = NULL,		/* no rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switcher */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = geostar_control_send,/* how to send commands */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = geostar_ntp_offset,
 #endif /* NTPSHM_ENABLE */
