@@ -77,7 +77,7 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
     if (data_len != 76)
 	return 0;
 
-    mask = ONLINE_IS;
+    mask = ONLINE_SET;
     gpsd_report(LOG_IO, "oncore NAVSOL - navigation data\n");
 
     flags = (unsigned char)getub(buf, 72);
@@ -94,7 +94,7 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
 	session->newdata.mode = MODE_NO_FIX;
 	session->gpsdata.status = STATUS_NO_FIX;
     }
-    mask |= MODE_IS;
+    mask |= MODE_SET;
     /*@ +predboolothers @*/
 
     /* Unless we have seen non-zero utc offset data, the time is GPS time
@@ -112,7 +112,7 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
 	/*@ -unrecog */
 	session->newdata.time = (timestamp_t)timegm(&unpacked_date) + nsec * 1e-9;
 	/*@ +unrecog */
-	mask |= TIME_IS;
+	mask |= TIME_SET;
 	gpsd_report(LOG_IO,
 		    "oncore NAVSOL - time: %04d-%02d-%02d %02d:%02d:%02d.%09d\n",
 		    unpacked_date.tm_year + 1900, unpacked_date.tm_mon + 1,
@@ -143,9 +143,7 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
     session->newdata.speed = speed;
     session->newdata.track = track;
 
-    mask |= LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS;
-
-    session->driver.oncore.good_time = 0;
+    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET;
 
     gpsd_zero_satellites(&session->gpsdata);
     /* Merge the satellite information from the Bb message. */
@@ -176,10 +174,18 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
 		    break;
 		}
 	    st++;
+	    /* bit 7 of the status word: sat used for position */  
 	    if (status & 0x80)
 		session->gpsdata.used[nsv++] = sv;
+	    /* bit 2 of the status word: using for time solution */
 	    if (status & 0x02)
-		session->driver.oncore.good_time = 1;
+		mask |= PPSTIME_IS;
+	    /*
+	     * The PPSTIME_IS mask bit exists distinctly from TIME_SET exactly
+	     * so an OnCore running in time-service mode (and other GPS clocks)
+	     * can signal that it's returning time even though no position fixes
+	     * have been available.
+	     */
 	}
     }
     for (j = 0; (int)j < session->driver.oncore.visible; j++)
@@ -196,7 +202,7 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
     session->gpsdata.satellites_used = (int)nsv;
     session->gpsdata.satellites_visible = (int)st;
 
-    mask |= SATELLITE_IS | USED_IS;
+    mask |= SATELLITE_SET | USED_IS;
 
     /* Some messages can only be polled.  As they are not so
      * important, would be enough to poll e.g. one message per cycle.
@@ -208,13 +214,13 @@ oncore_msg_navsol(struct gps_device_t *session, unsigned char *buf,
     (void)oncore_control_send(session, (char *)pollEn, sizeof(pollEn));
 
     gpsd_report(LOG_DATA,
-		"NAVSOL: time=%.2f lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f mode=%d status=%d visible=%d used=%d mask=%s\n",
+		"NAVSOL: time=%.2f lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f mode=%d status=%d visible=%d used=%d\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
 		session->newdata.speed, session->newdata.track,
 		session->newdata.mode, session->gpsdata.status,
 		session->gpsdata.satellites_used,
-		session->gpsdata.satellites_visible, gpsd_maskdump(mask));
+		session->gpsdata.satellites_visible);
     return mask;
 }
 
@@ -304,7 +310,7 @@ oncore_msg_svinfo(struct gps_device_t *session, unsigned char *buf,
     }
 
     gpsd_report(LOG_DATA, "SVINFO: mask={SATELLITE}\n");
-    return SATELLITE_IS;
+    return SATELLITE_SET;
 }
 
 /**
@@ -354,8 +360,7 @@ gps_mask_t oncore_dispatch(struct gps_device_t * session, unsigned char *buf,
     type = ONCTYPE(buf[2], buf[3]);
 
     /* we may need to dump the raw packet */
-    gpsd_report(LOG_RAW, "raw oncore packet type 0x%04x length %zd: %s\n",
-		type, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
+    gpsd_report(LOG_RAW, "raw Oncore packet type 0x%04x\n", type);
 
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		   "MOT-%c%c", type >> 8, type & 0xff);
@@ -382,10 +387,8 @@ gps_mask_t oncore_dispatch(struct gps_device_t * session, unsigned char *buf,
 
     default:
 	/* FIX-ME: This gets noisy in a hurry. Change once your driver works */
-	gpsd_report(LOG_WARN, "unknown packet id @@%c%c length %zd: %s\n",
-		    type >> 8, type & 0xff, len, gpsd_hexdump_wrapper(buf,
-								      len,
-								      LOG_WARN));
+	gpsd_report(LOG_WARN, "unknown packet id @@%c%c length %zd\n",
+		    type >> 8, type & 0xff, len);
 	return 0;
     }
 }
@@ -418,10 +421,7 @@ static ssize_t oncore_control_send(struct gps_device_t *session,
     session->msgbuf[msglen + 4] = '\n';
     session->msgbuflen = msglen + 5;
 
-    gpsd_report(LOG_IO, "writing oncore control type %c%c:%s\n",
-		msg[0], msg[1], gpsd_hexdump_wrapper(session->msgbuf,
-						     session->msgbuflen,
-						     LOG_IO));
+    gpsd_report(LOG_IO, "writing oncore control type %c%c\n", msg[0], msg[1]);
     return gpsd_write(session, session->msgbuf, session->msgbuflen);
 }
 
@@ -466,7 +466,7 @@ static double oncore_ntp_offset(struct gps_device_t *session UNUSED)
 }
 #endif /* NTPSHM_ENABLE */
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static bool oncore_set_speed(struct gps_device_t *session UNUSED,
 			     speed_t speed UNUSED,
 			     char parity UNUSED, int stopbits UNUSED)
@@ -499,7 +499,7 @@ static void oncore_set_mode(struct gps_device_t *session, int mode)
 	session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 static gps_mask_t oncore_parse_input(struct gps_device_t *session)
 {
@@ -534,16 +534,16 @@ const struct gps_type_t oncore_binary = {
     .parse_packet     = oncore_parse_input,	/* packet parser */
     .rtcm_writer      = gpsd_write,		/* device accepts RTCM */
     .event_hook     = oncore_event_hook,	/* lifetime event hook */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher   = oncore_set_speed,	/* no speed setter */
     .mode_switcher    = oncore_set_mode,	/* no mode setter */
     .rate_switcher    = NULL,			/* no speed setter */
     .min_cycle        = 1,			/* 1Hz */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     /* Control string sender - should provide checksum and headers/trailer */
     .control_send   = oncore_control_send,	/* to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset = oncore_ntp_offset,		/* NTP offset array */
 #endif /* NTPSHM_ENABLE */
