@@ -46,7 +46,7 @@
 #define HI(n)		((n) >> 8)
 #define LO(n)		((n) & 0xff)
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 /*@ +charint @*/
 /* message to enable:
  *   MID 7 Clock Status
@@ -117,7 +117,7 @@ static unsigned char enablemid52[] = {
 };
 
 /*@ -charint @*/
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 
 static gps_mask_t sirf_msg_debug(unsigned char *, size_t);
@@ -164,7 +164,7 @@ static bool sirf_write(struct gps_device_t *session, unsigned char *msg)
     return (ok);
 }
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 static ssize_t sirf_control_send(struct gps_device_t *session, char *msg,
 				 size_t len)
 {
@@ -184,9 +184,9 @@ static ssize_t sirf_control_send(struct gps_device_t *session, char *msg,
     /* *INDENT-ON* */
     /*@ -charint -matchanyintegral +initallelements +mayaliasunique @*/
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static bool sirfbin_speed(struct gps_device_t *session, speed_t speed, char parity, int stopbits)
 /* change speed in binary mode */
 {
@@ -290,7 +290,7 @@ static void sirfbin_mode(struct gps_device_t *session, int mode)
     }
     session->back_to_nmea = false;
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 static ssize_t sirf_get(struct gps_device_t *session)
 {
@@ -376,8 +376,8 @@ static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session UNUSED,
      * different double orders, neither IEEE754 */
     /* FIXME - decode the time, since this is the first MID with a
      * good time stamp this will be good for ntpshm time */
-    gpsd_report(LOG_PROG, "SiRF: MID 0x1c, NLMD, gps_tow: %f, %s\n",
-		(double)gps_tow, gpsd_hexdump_wrapper(&gps_tow, 8, LOG_PROG));
+    gpsd_report(LOG_PROG, "SiRF: MID 0x1c, NLMD, gps_tow: %f\n", 
+		(double)gps_tow);
 
     return 0;
 }
@@ -405,12 +405,12 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
     } else {
 	session->driver.sirf.driverstate |= SIRF_GE_232;
     }
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly) {
 	gpsd_report(LOG_PROG, "SiRF: Enabling PPS message...\n");
 	(void)sirf_write(session, enablemid52);
     }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
     if (strstr((char *)(buf + 1), "ES"))
 	gpsd_report(LOG_INF, "SiRF: Firmware has XTrac capability\n");
@@ -419,16 +419,16 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
 #ifdef NTPSHM_ENABLE
     session->driver.sirf.time_seen = 0;
 #endif /* NTPSHM_ENABLE */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly && session->gpsdata.dev.baudrate >= 38400) {
         /* some USB devices are also too slow, no way to tell which ones */
 	gpsd_report(LOG_PROG, "SiRF: Enabling subframe transmission...\n");
 	(void)sirf_write(session, enablesubframe);
     }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
     gpsd_report(LOG_DATA, "SiRF: FV MID 0x06: subtype='%s' mask={DEVICEID}\n",
 		session->subtype);
-    return DEVICEID_IS;
+    return DEVICEID_SET;
 }
 
 static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
@@ -447,15 +447,16 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
 	words[i] = (uint32_t)getbeu32(buf, 4 * i + 3);
     }
 
+    gpsd_report(LOG_IO, "SiRF: NavData chan %u svid %u\n",chan,svid);
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly && session->gpsdata.dev.baudrate < 38400) {
         /* some USB are also too slow, no way to tell which ones */
 	gpsd_report(LOG_WARN, 
 		"WARNING: SiRF: link too slow, disabling subframes.\n");
 	(void)sirf_write(session, disablesubframe);
     }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
     return gpsd_interpret_subframe_raw(session, svid, words);
 }
@@ -505,21 +506,22 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
     }
     session->gpsdata.satellites_visible = st;
 #ifdef NTPSHM_ENABLE
-    if (st <= 3) {
+    if (st < 3) {
 	gpsd_report(LOG_PROG,
 		    "SiRF: NTPD not enough satellites seen: %d\n", st);
     } else {
 	/* SiRF says if 3 sats in view the time is good */
-	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_GPS_1)) {
-	    gpsd_report(LOG_RAW, "SiRF: NTPD just seen GPS_1\n");
-	}
 	gpsd_report(LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x04, seen=0x%02x, time:%.2lf, leap:%d\n",
 		    session->driver.sirf.time_seen,
 		    session->gpsdata.skyview_time,
 		    session->context->leap_seconds);
-	session->driver.sirf.time_seen |= TIME_SEEN_GPS_1;
-	mask |= TIME_IS;
+	/*
+	 * Might be better if we didn't change the fix timestamp here,
+	 * but presently there's no other way to pass the time to NTP.
+	 */
+	session->newdata.time = session->gpsdata.skyview_time;
+	mask |= TIME_SET | PPSTIME_IS;
 	/*
 	 * This time stamp, at 4800bps, is so close to 1 sec old as to
 	 * be confusing to ntpd, but ntpshm_put() will ignore it if a better
@@ -529,7 +531,7 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 #endif /* NTPSHM_ENABLE */
     gpsd_report(LOG_DATA, "SiRF: MTD 0x04: visible=%d mask={SATELLITE}\n",
 		session->gpsdata.satellites_visible);
-    return SATELLITE_IS | mask;
+    return SATELLITE_SET | mask;
 }
 
 #ifdef NTPSHM_ENABLE
@@ -623,7 +625,7 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
     else if (session->gpsdata.status != 0)
 	session->newdata.mode = MODE_2D;
     if (session->newdata.mode == MODE_3D)
-	mask |= ALTITUDE_IS | CLIMB_IS;
+	mask |= ALTITUDE_SET | CLIMB_SET;
     gpsd_report(LOG_PROG,
 		"SiRF: MND 0x02: Navtype = 0x%0x, Status = %d, mode = %d\n",
 		navtype, session->gpsdata.status, session->newdata.mode);
@@ -636,29 +638,27 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
 	gpsd_report(LOG_PROG, "SiRF: NTPD no fix, mode: %d\n",
 		    session->newdata.mode);
     } else {
-	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_GPS_2)) {
-	    gpsd_report(LOG_PROG, "SiRF: NTPD SEEN_GPS_2\n");
-	}
 	gpsd_report(LOG_PROG,
 		    "SiRF: NTPD valid time MID 0x02, seen=0x%02x, time;%.2lf, leap:%d\n",
 		    session->driver.sirf.time_seen,
 		    session->newdata.time, session->context->leap_seconds);
-	session->driver.sirf.time_seen |= TIME_SEEN_GPS_2;
     }
 #endif /* NTPSHM_ENABLE */
     /* fix quality data */
     session->gpsdata.dop.hdop = (double)getub(buf, 20) / 5.0;
     mask |=
-	TIME_IS | LATLON_IS | ALTITUDE_IS | TRACK_IS | SPEED_IS | STATUS_IS |
-	MODE_IS | DOP_IS | USED_IS;
+	TIME_SET | LATLON_SET | ALTITUDE_SET | TRACK_SET |
+	SPEED_SET | STATUS_SET | MODE_SET | DOP_SET | USED_IS;
+    if ( 3 <= session->gpsdata.satellites_visible ) {
+	mask |= PPSTIME_IS;
+    }
     gpsd_report(LOG_DATA,
-		"SiRF: MND 0x02: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d hdop=%.2f used=%d mask=%s\n",
+		"SiRF: MND 0x02: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d hdop=%.2f used=%d\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
 		session->newdata.track, session->newdata.speed,
 		session->newdata.mode, session->gpsdata.status,
-		session->gpsdata.dop.hdop, session->gpsdata.satellites_used,
-		gpsd_maskdump(mask));
+		session->gpsdata.dop.hdop, session->gpsdata.satellites_used);
     return mask;
 }
 
@@ -715,21 +715,21 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
     gpsd_report(LOG_PROG,
 		"SiRF: GND 0x29: Navtype = 0x%0x, Status = %d, mode = %d\n",
 		navtype, session->gpsdata.status, session->newdata.mode);
-    mask |= STATUS_IS | MODE_IS;
+    mask |= STATUS_SET | MODE_SET;
 
     session->newdata.latitude = getbes32(buf, 23) * 1e-7;
     session->newdata.longitude = getbes32(buf, 27) * 1e-7;
     if (session->newdata.latitude != 0 && session->newdata.latitude != 0)
-	mask |= LATLON_IS;
+	mask |= LATLON_SET;
 
     if ((eph = getbes32(buf, 50) * 1e-2) > 0) {
 	session->newdata.epx = session->newdata.epy = eph / sqrt(2);
-	mask |= HERR_IS;
+	mask |= HERR_SET;
     }
     if ((session->newdata.epv = getbes32(buf, 54) * 1e-2) > 0)
-	mask |= VERR_IS;
+	mask |= VERR_SET;
     if ((session->newdata.eps = getbes16(buf, 62) * 1e-2) > 0)
-	mask |= SPEEDERR_IS;
+	mask |= SPEEDERR_SET;
 
     /* HDOP should be available at byte 89, but in 231 it's zero. */
     //session->gpsdata.dop.hdop = (unsigned int)getub(buf, 89) * 0.2;
@@ -790,13 +790,12 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	    gpsd_report(LOG_PROG, "SiRF: NTPD no year\n",
 			session->newdata.mode);
 	} else {
-	    if (0 == (session->driver.sirf.time_seen & TIME_SEEN_UTC_1)) {
-		gpsd_report(LOG_RAW, "SiRF: NTPD just SEEN_UTC 1\n");
-	    }
 	    gpsd_report(LOG_PROG,
 			"SiRF: NTPD valid time MID 0x29, seen=0x%02x\n",
 			session->driver.sirf.time_seen);
-	    session->driver.sirf.time_seen |= TIME_SEEN_UTC_1;
+	}
+	if ( 3 <= session->gpsdata.satellites_visible ) {
+	    mask |= PPSTIME_IS;
 	}
 
 #endif /* NTPSHM_ENABLE */
@@ -807,12 +806,12 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	session->newdata.track = getbeu16(buf, 42) * 1e-2;
 	/* skip 2 bytes of magnetic variation */
 	session->newdata.climb = getbes16(buf, 46) * 1e-2;
-	mask |= TIME_IS | SPEED_IS | TRACK_IS;
+	mask |= TIME_SET | SPEED_SET | TRACK_SET;
 	if (session->newdata.mode == MODE_3D)
-	    mask |= ALTITUDE_IS | CLIMB_IS;
+	    mask |= ALTITUDE_SET | CLIMB_SET;
     }
     gpsd_report(LOG_DATA,
-		"SiRF: GND 0x29: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d mask=%s\n",
+		"SiRF: GND 0x29: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f mode=%d status=%d\n",
 		session->newdata.time,
 		session->newdata.latitude,
 		session->newdata.longitude,
@@ -820,7 +819,7 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 		session->newdata.track,
 		session->newdata.speed,
 		session->newdata.mode,
-		session->gpsdata.status, gpsd_maskdump(mask));
+		session->gpsdata.status);
     return mask;
 }
 #endif /* __UNUSED__ */
@@ -841,12 +840,12 @@ static gps_mask_t sirf_msg_sysparam(struct gps_device_t *session,
     session->driver.sirf.degraded_timeout = (unsigned char)getub(buf, 10);
     session->driver.sirf.dr_timeout = (unsigned char)getub(buf, 11);
     session->driver.sirf.track_smooth_mode = (unsigned char)getub(buf, 12);
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly) {
 	gpsd_report(LOG_PROG, "SiRF: Setting Navigation Parameters\n");
 	(void)sirf_write(session, modecontrol);
     }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
     return 0;
 }
 
@@ -860,8 +859,8 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 	return 0;
 
     /* this packet is only sent by uBlox firmware from version 1.32 */
-    mask = LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | CLIMB_IS |
-	STATUS_IS | MODE_IS | DOP_IS;
+    mask = LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET |
+	STATUS_SET | MODE_SET | DOP_SET;
     session->newdata.latitude = (double)getbes32(buf, 1) * RAD_2_DEG * 1e-8;
     session->newdata.longitude = (double)getbes32(buf, 5) * RAD_2_DEG * 1e-8;
     session->gpsdata.separation =
@@ -891,7 +890,10 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
     if (navtype & 0x40) {	/* UTC corrected timestamp? */
 	struct tm unpacked_date;
 	double subseconds;
-	mask |= TIME_IS;
+	mask |= TIME_SET;
+	if ( 3 <= session->gpsdata.satellites_visible ) {
+	    mask |= PPSTIME_IS;
+	}
 	unpacked_date.tm_year = (int)getbeu16(buf, 26) - 1900;
 	unpacked_date.tm_mon = (int)getub(buf, 28) - 1;
 	unpacked_date.tm_mday = (int)getub(buf, 29);
@@ -921,15 +923,14 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
     session->gpsdata.dop.tdop = (int)getub(buf, 38) / 5.0;
     session->driver.sirf.driverstate |= UBLOX;
     gpsd_report(LOG_DATA,
-		"SiRF: EMD 0x62: time=%.2f lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mask=%s\n",
+		"SiRF: EMD 0x62: time=%.2f lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
 		session->newdata.speed, session->newdata.track,
 		session->newdata.climb, session->newdata.mode,
 		session->gpsdata.status, session->gpsdata.dop.gdop,
 		session->gpsdata.dop.pdop, session->gpsdata.dop.hdop,
-		session->gpsdata.dop.vdop, session->gpsdata.dop.tdop,
-		gpsd_maskdump(mask));
+		session->gpsdata.dop.vdop, session->gpsdata.dop.tdop);
     return mask;
 }
 
@@ -965,7 +966,10 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
 		    session->driver.sirf.time_seen);
 	session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
 #endif /* NTPSHM_ENABLE */
-	mask |= TIME_IS;
+	mask |= TIME_SET;
+	if ( 3 <= session->gpsdata.satellites_visible ) {
+	    mask |= PPSTIME_IS;
+	}
     }
     return mask;
 }
@@ -979,8 +983,7 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 
     buf += 4;
     len -= 8;
-    gpsd_report(LOG_RAW, "SiRF: Raw packet type 0x%02x length %zd: %s\n",
-		buf[0], len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
+    gpsd_report(LOG_RAW, "SiRF: Raw packet type 0x%02x\n", buf[0]);
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		   "MID%d", (int)buf[0]);
 
@@ -1049,18 +1052,15 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return 0;
 
     case 0x0e:			/* Almanac Data MID 14 */
-	gpsd_report(LOG_PROG, "SiRF: unused ALM  0x0e: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused ALM  0x0e\n");
 	return 0;
 
     case 0x0f:			/* Ephemeris Data MID 15 */
-	gpsd_report(LOG_PROG, "SiRF: unused EPH  0x0f: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused EPH  0x0f\n");
 	return 0;
 
     case 0x11:			/* Differential Corrections MID 17 */
-	gpsd_report(LOG_PROG, "SiRF: unused DIFF 0x11: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused DIFF 0x11\n");
 	return 0;
 
     case 0x12:			/* OK To Send MID 18 */
@@ -1072,38 +1072,31 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return sirf_msg_sysparam(session, buf, len);
 
     case 0x1b:			/* DGPS status (undocumented) MID 27 */
-	gpsd_report(LOG_PROG, "SiRF: unused DGPSF 0x1b %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused DGPSF 0x1b\n");
 	return 0;
 
     case 0x1c:			/* Navigation Library Measurement Data MID 28 */
-	gpsd_report(LOG_PROG, "SiRF: NLMD 0x1c: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: NLMD 0x1c\n");
 	return sirf_msg_nlmd(session, buf, len);
 
     case 0x1d:			/* Navigation Library DGPS Data MID 29 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLDG 0x1d: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused NLDG 0x1d\n");
 	return 0;
 
     case 0x1e:			/* Navigation Library SV State Data MID 30 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLSV 0x1e: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused NLSV 0x1e\n");
 	return 0;
 
     case 0x1f:			/* Navigation Library Initialization Data MID 31 */
-	gpsd_report(LOG_PROG, "SiRF: unused NLID 0x1f: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused NLID 0x1f\n");
 	return 0;
 
     case 0x29:			/* Geodetic Navigation Information MID 41 */
-	gpsd_report(LOG_PROG, "SiRF: unused GND 0x29: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused GND 0x29\n");
 	return 0;
 
     case 0x32:			/* SBAS corrections MID 50 */
-	gpsd_report(LOG_PROG, "SiRF: unused SBAS 0x32: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused SBAS 0x32\n");
 	return 0;
 
     case 0x34:			/* PPS Time MID 52 */
@@ -1131,13 +1124,11 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return sirf_msg_ppstime(session, buf, len);
 
     case 0x62:			/* uBlox Extended Measured Navigation Data MID 98 */
-	gpsd_report(LOG_PROG, "SiRF: uBlox EMND 0x62: %s.\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: uBlox EMND 0x62\n");
 	return sirf_msg_ublox(session, buf, len) | (CLEAR_IS | REPORT_IS);
 
     case 0x80:			/* Initialize Data Source MID 128 */
-	gpsd_report(LOG_PROG, "SiRF: unused INIT 0x80: %s\n",
-		    gpsd_hexdump_wrapper(buf, len, LOG_PROG));
+	gpsd_report(LOG_PROG, "SiRF: unused INIT 0x80\n");
 	return 0;
 
     case 0xe1:			/* Development statistics messages MID 225 */
@@ -1147,8 +1138,8 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	return 0;
 
     default:
-	gpsd_report(LOG_WARN, "SiRF: Unknown packet id %d length %zd: %s\n",
-		    buf[0], len, gpsd_hexdump_wrapper(buf, len, LOG_WARN));
+	gpsd_report(LOG_WARN, "SiRF: Unknown packet id %d length %zd\n", 
+		    buf[0], len);
 	return 0;
     }
 }
@@ -1344,15 +1335,15 @@ const struct gps_type_t sirf_binary =
     .parse_packet   = sirfbin_parse_input,/* parse message packets */
     .rtcm_writer    = gpsd_write,	/* send RTCM data straight */
     .event_hook     = sirfbin_event_hook,/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = sirfbin_speed,	/* we can change baud rate */
     .mode_switcher  = sirfbin_mode,	/* there's a mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = sirf_control_send,/* how to send a control string */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = sirf_ntp_offset,
 #endif /* NTP_SHM_ENABLE */
