@@ -30,21 +30,7 @@ gps_mask_t generic_parse_input(struct gps_device_t *session)
     const struct gps_type_t **dp;
 
     if (session->packet.type == COMMENT_PACKET) {
-	unsigned char *cp;
-	int year;
-	/*
-	 * Interpret "Date: dd mmm yyyy", setting the session context
-	 * century from the year.  We do this so the behavior of the
-	 * regression tests won't depend on what century the daemon
-	 * started up in.
-	 */
-	if (strstr((char *)session->packet.outbuffer, "Date:") != NULL) {
-	    cp = session->packet.outbuffer + strlen((char *)session->packet.outbuffer) - 1;
-	    while (isspace(*cp))
-		--cp;
-	    year = atoi((char *)cp - 4);
-	    session->context->century = year - (year % 100);
-	}
+	gpsd_set_century(session);
 	return 0;
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
@@ -70,7 +56,7 @@ gps_mask_t generic_parse_input(struct gps_device_t *session)
 			&& session->device_type->event_hook != NULL)
 			session->device_type->event_hook(session,
 							 event_triggermatch);
-		    st |= DEVICEID_IS;
+		    st |= DEVICEID_SET;
 		}
 	    }
 	}
@@ -105,15 +91,15 @@ const struct gps_type_t unknown = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* write RTCM data straight */
     .event_hook     = NULL,		/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = NULL,		/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -246,10 +232,9 @@ static void nmea_event_hook(struct gps_device_t *session, event_t event)
     }
 }
 
-#ifdef ALLOW_RECONFIGURE
+#if defined(RECONFIGURE_ENABLE) && defined(BINARY_ENABLE)
 static void nmea_mode_switch(struct gps_device_t *session, int mode)
 {
-#ifdef BINARY_ENABLE
     /*
      * If the daemon has seen this device in a binary mode, we may
      * actually know how to switch back.
@@ -260,7 +245,7 @@ static void nmea_mode_switch(struct gps_device_t *session, int mode)
 
 	/*@-shiftnegative@*/
 	for (dp = gpsd_drivers; *dp; dp++) {
-	    if ((*dp)->packet_type > 0 &&
+	    if ((*dp)->packet_type > 0 && (*dp)->packet_type != session->packet.type &&
 	    	    (session->observed & PACKET_TYPEMASK((*dp)->packet_type))!=0) { 
 		(*dp)->mode_switcher(session, mode);
 		break;
@@ -268,9 +253,8 @@ static void nmea_mode_switch(struct gps_device_t *session, int mode)
 	}
 	/*@+shiftnegative@*/
     }
-#endif /* BINARY_ENABLE */
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* defined(RECONFIGURE_ENABLE) && defined(BINARY_ENABLE) */
 
 /* *INDENT-OFF* */
 const struct gps_type_t nmea = {
@@ -284,15 +268,19 @@ const struct gps_type_t nmea = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = gpsd_write,	/* write RTCM data straight */
     .event_hook     = nmea_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
+#ifdef BINARY_ENABLE
     .mode_switcher  = nmea_mode_switch,	/* maybe switchable if it was a SiRF */
+#else
+    .mode_switcher  = NULL,		/* no binary mode to revert to */
+#endif /* BINARY_ENABLE */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -306,7 +294,7 @@ const struct gps_type_t nmea = {
  *
  **************************************************************************/
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static void garmin_mode_switch(struct gps_device_t *session, int mode)
 /* only does anything in one direction, going to Garmin binary driver */
 {
@@ -317,7 +305,7 @@ static void garmin_mode_switch(struct gps_device_t *session, int mode)
 	session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 static void garmin_nmea_event_hook(struct gps_device_t *session,
 				   event_t event)
@@ -386,15 +374,15 @@ const struct gps_type_t garmin = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* some do, some don't, skip for now */
     .event_hook     = garmin_nmea_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,			/* no speed switcher */
     .mode_switcher  = garmin_mode_switch,	/* mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /*ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /*RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -448,15 +436,15 @@ const struct gps_type_t ashtech = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = gpsd_write,	/* write RTCM data straight */
     .event_hook     = ashtech_event_hook, /* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -499,15 +487,15 @@ const struct gps_type_t fv18 = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = gpsd_write,	/* write RTCM data straight */
     .event_hook     = fv18_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -552,15 +540,15 @@ const struct gps_type_t gpsclock = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = gpsd_write,	/* write RTCM data straight */
     .event_hook     = gpsclock_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* sample rate is fixed */
     .min_cycle      = 1,		/* sample rate is fixed */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -607,15 +595,15 @@ static const struct gps_type_t tripmate = {
     .parse_packet  = generic_parse_input,		/* how to interpret a packet */
     .rtcm_writer   = gpsd_write,			/* send RTCM data straight */
     .event_hook    = tripmate_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher= NULL,			/* no speed switcher */
     .mode_switcher = NULL,			/* no mode switcher */
     .rate_switcher = NULL,			/* no sample-rate switcher */
     .min_cycle     = 1,				/* no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send  = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -658,15 +646,15 @@ static const struct gps_type_t earthmate = {
     .parse_packet  = generic_parse_input,		/* how to interpret a packet */
     .rtcm_writer   = NULL,			/* don't send RTCM data */
     .event_hook    = earthmate_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher= NULL,			/* no speed switcher */
     .mode_switcher = NULL,			/* no mode switcher */
     .rate_switcher = NULL,			/* no sample-rate switcher */
     .min_cycle     = 1,				/* no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send  = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
@@ -742,7 +730,7 @@ static bool tnt_send(struct gps_device_t *session, const char *fmt, ...)
     }
 }
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static bool tnt_speed(struct gps_device_t *session,
 		      speed_t speed, char parity UNUSED, int stopbits UNUSED)
 {
@@ -760,7 +748,7 @@ static bool tnt_speed(struct gps_device_t *session,
     return tnt_send(session, "@B6=%d", i + 1)
 	&& tnt_send(session, "@F28.6=1");
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 static void tnt_event_hook(struct gps_device_t *session, event_t event)
 /* TNT lifetime event hook */
@@ -785,15 +773,15 @@ const struct gps_type_t trueNorth = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* Don't send */
     .event_hook     = tnt_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = tnt_speed,	/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no wrapup */
     .min_cycle      = 0.5,		/* fixed at 20 samples per second */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = tnt_control_send,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -861,15 +849,15 @@ static const struct gps_type_t oceanServer = {
     .parse_packet   = generic_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* Don't send */
     .event_hook     = oceanserver_event_hook,
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no wrapup */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -887,15 +875,17 @@ static const struct gps_type_t oceanServer = {
 static gps_mask_t rtcm104v2_analyze(struct gps_device_t *session)
 {
     rtcm2_unpack(&session->gpsdata.rtcm2, (char *)session->packet.isgps.buf);
-    gpsd_report(LOG_RAW, "RTCM 2.x packet type 0x%02x length %d words from %zd bytes: %s\n",
-		session->gpsdata.rtcm2.type,
-		session->gpsdata.rtcm2.length + 2,
-		session->packet.isgps.buflen,
-		gpsd_hexdump_wrapper(session->packet.isgps.buf,
-				     (session->gpsdata.rtcm2.length +
-				      2) * sizeof(isgps30bits_t), LOG_RAW));
+    /* extra guard prevents expensive hexdump calls */
+    if (session->context->debug >= LOG_RAW)
+	gpsd_report(LOG_RAW, "RTCM 2.x packet type 0x%02x length %d words from %zd bytes: %s\n",
+		    session->gpsdata.rtcm2.type,
+		    session->gpsdata.rtcm2.length + 2,
+		    session->packet.isgps.buflen,
+		    gpsd_hexdump((char *)session->packet.isgps.buf,
+				 (session->gpsdata.rtcm2.length +
+				  2) * sizeof(isgps30bits_t)));
     session->cycle_end_reliable = true;
-    return RTCM2_IS;
+    return RTCM2_SET;
 }
 
 /* *INDENT-OFF* */
@@ -910,15 +900,15 @@ static const struct gps_type_t rtcm104v2 = {
     .parse_packet  = rtcm104v2_analyze,	/*  */
     .rtcm_writer   = NULL,		/* don't send RTCM data,  */
     .event_hook    = NULL,		/* no event_hook */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher= NULL,		/* no speed switcher */
     .mode_switcher = NULL,		/* no mode switcher */
     .rate_switcher = NULL,		/* no sample-rate switcher */
     .min_cycle     = 1,			/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -936,15 +926,10 @@ static gps_mask_t rtcm104v3_analyze(struct gps_device_t *session)
 {
     uint16_t type = getbeu16(session->packet.inbuffer, 3) >> 4;
 
-    gpsd_report(LOG_RAW, "RTCM 3.x packet %d type length %zd bytes: %s\n",
-		type,
-		session->packet.outbuflen, 
-		gpsd_hexdump_wrapper(session->packet.outbuffer,
-				     session->packet.outbuflen,
-				     LOG_RAW));
+    gpsd_report(LOG_RAW, "RTCM 3.x packet %d\n", type);
     rtcm3_unpack(&session->gpsdata.rtcm3, (char *)session->packet.outbuffer);
     session->cycle_end_reliable = true;
-    return RTCM3_IS;
+    return RTCM3_SET;
 }
 
 /* *INDENT-OFF* */
@@ -959,15 +944,15 @@ static const struct gps_type_t rtcm104v3 = {
     .parse_packet  = rtcm104v3_analyze,	/*  */
     .rtcm_writer   = NULL,		/* don't send RTCM data,  */
     .event_hook    = NULL,		/* no event hook */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher= NULL,		/* no speed switcher */
     .mode_switcher = NULL,		/* no mode switcher */
     .rate_switcher = NULL,		/* no sample-rate switcher */
     .min_cycle     = 1,			/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -994,15 +979,15 @@ static const struct gps_type_t garmintxt = {
     .parse_packet  = generic_parse_input,	/* how to parse one */
     .rtcm_writer   = NULL,		/* don't send RTCM data,  */
     .event_hook    = NULL,		/* no event hook */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher= NULL,		/* no speed switcher */
     .mode_switcher = NULL,		/* no mode switcher */
     .rate_switcher = NULL,		/* no sample-rate switcher */
     .min_cycle     = 1,			/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -1031,7 +1016,7 @@ static gps_mask_t processMTK3301(struct gps_device_t *session)
     mask = generic_parse_input(session);
 
     if (session->packet.type == NMEA_PACKET 
-	&& strncmp(session->driver.nmea.field[0], "$PMTK", 5) == 0)
+	&& strncmp(session->driver.nmea.field[0], "PMTK", 4) == 0)
     {
 	msg = atoi(&(session->driver.nmea.field[0])[4]);
 	switch (msg) {
@@ -1039,7 +1024,7 @@ static gps_mask_t processMTK3301(struct gps_device_t *session)
 	    (void)strlcat(session->subtype, session->driver.nmea.field[1], sizeof(session->subtype));
 	    (void)strlcat(session->subtype, "-", sizeof(session->subtype));
 	    (void)strlcat(session->subtype, session->driver.nmea.field[2], sizeof(session->subtype));
-	    return ONLINE_IS;
+	    return ONLINE_SET;
 	case 001:			/* ACK / NACK */
 	    reason = atoi(session->driver.nmea.field[2]);
 	    if (atoi(session->driver.nmea.field[1]) == -1)
@@ -1051,7 +1036,7 @@ static gps_mask_t processMTK3301(struct gps_device_t *session)
 		gpsd_report(LOG_WARN, "MTK ACK: %s\n", session->driver.nmea.field[1]);
 	    break;
 	default:
-	    return ONLINE_IS;		/* ignore */
+	    return ONLINE_SET;		/* ignore */
 	}
     }
 
@@ -1092,7 +1077,7 @@ static void mtk3301_event_hook(struct gps_device_t *session, event_t event)
     }
 }
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 static bool mtk3301_rate_switcher(struct gps_device_t *session, double rate)
 {
     char buf[78];
@@ -1107,7 +1092,7 @@ static bool mtk3301_rate_switcher(struct gps_device_t *session, double rate)
     (void)nmea_send(session, buf);	/* Fix interval */
     return true;
 }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 /* *INDENT-OFF* */
 const struct gps_type_t mtk3301 = {
@@ -1121,15 +1106,15 @@ const struct gps_type_t mtk3301 = {
     .parse_packet   = processMTK3301,	/* how to interpret a packet */
     .rtcm_writer    = gpsd_write,	/* write RTCM data straight */
     .event_hook     = mtk3301_event_hook,	/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = mtk3301_rate_switcher,		/* sample rate switcher */
     .min_cycle      = 0.2,		/* max 5Hz */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = nmea_write,	/* how to send control strings */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,
 #endif /* NTPSHM_ ENABLE */
@@ -1137,7 +1122,6 @@ const struct gps_type_t mtk3301 = {
 /* *INDENT-ON* */
 #endif /* MTK3301_ENABLE */
 
-
 #ifdef AIVDM_ENABLE
 /**************************************************************************
  *
@@ -1150,10 +1134,10 @@ static gps_mask_t aivdm_analyze(struct gps_device_t *session)
     if (session->packet.type == AIVDM_PACKET) {
 	if (aivdm_decode
 	    ((char *)session->packet.outbuffer, session->packet.outbuflen,
-	     session->aivdm, &session->gpsdata.ais)) {
-	    return ONLINE_IS | AIS_IS;
+	     session->aivdm, &session->gpsdata.ais, session->context->debug)) {
+	    return ONLINE_SET | AIS_SET;
 	} else
-	    return ONLINE_IS;
+	    return ONLINE_SET;
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
 	return nmea_parse((char *)session->packet.outbuffer, session);
@@ -1175,15 +1159,15 @@ static const struct gps_type_t aivdm = {
     .parse_packet     = aivdm_analyze,	/* how to analyze a packet */
     .rtcm_writer      = NULL,		/* don't send RTCM data,  */
     .event_hook       = NULL,		/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher   = NULL,		/* no speed switcher */
     .mode_switcher    = NULL,		/* no mode switcher */
     .rate_switcher    = NULL,		/* no rate switcher */
     .min_cycle        = 1,		/* max 1Hz */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send     = NULL,		/* no control sender */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no NTP communication */
 #endif /* NTPSHM_ ENABLE */
@@ -1191,6 +1175,104 @@ static const struct gps_type_t aivdm = {
 /* *INDENT-ON* */
 #endif /* AIVDM_ENABLE */
 
+#ifdef PASSTHROUGH_ENABLE
+/**************************************************************************
+ *
+ * JSON passthrough driver
+ *
+ **************************************************************************/
+
+static void path_rewrite(struct gps_device_t *session, char *prefix)
+/* prepend the session path to the value of a specified attribute */
+{
+    /* 
+     * Hack the packet to reflect its origin.  This code is supposed
+     * to insert the path naming the remote gpsd instance into the
+     * baginning of the path attribute, followed by a # to separate it
+     * from the device.
+     */
+    char *prefloc;
+    for (prefloc = (char *)session->packet.outbuffer;
+	 prefloc < (char *)session->packet.outbuffer+session->packet.outbuflen;
+	 prefloc++)
+	if (strncmp(prefloc, prefix, strlen(prefix)) == 0) {
+	    char copy[sizeof(session->packet.outbuffer)];
+	    (void)strlcpy(copy, 
+			  (char *)session->packet.outbuffer, 
+			  sizeof(copy));
+	    prefloc += strlen(prefix);
+	    (void)strlcpy(prefloc,
+			  session->gpsdata.dev.path,
+			  sizeof(session->gpsdata.dev.path));
+	    (void)strlcat((char *)session->packet.outbuffer, "#", 
+			  sizeof(session->packet.outbuffer));
+	    (void)strlcat((char *)session->packet.outbuffer, 
+			  copy + (prefloc-(char *)session->packet.outbuffer), 
+			  sizeof(session->packet.outbuffer));
+	}
+    session->packet.outbuflen = strlen((char *)session->packet.outbuffer);
+}
+
+static gps_mask_t json_pass_packet(struct gps_device_t *session UNUSED)
+{
+    gpsd_report(LOG_IO, "<= GPS: %s\n", (char *)session->packet.outbuffer);
+
+    /*@-nullpass@*/ /* required only because splint is buggy */
+    /* devices and paths need to be edited to */
+    if (strstr((char *)session->packet.outbuffer, "DEVICE") != NULL)
+	path_rewrite(session, "\"path\":\"");
+    path_rewrite(session, "\"device\":\"");		     
+
+    /* mark certain responses without a path or device attribute */
+    if (strstr((char *)session->packet.outbuffer, "VERSION") != NULL
+	|| strstr((char *)session->packet.outbuffer, "WATCH") != NULL
+	|| strstr((char *)session->packet.outbuffer, "DEVICES") != NULL) {
+	session->packet.outbuffer[session->packet.outbuflen] = '\0';	
+	(void)strlcat((char *)session->packet.outbuffer, ",\"remote\":\"", 
+		      sizeof(session->packet.outbuffer));
+	(void)strlcat((char *)session->packet.outbuffer,
+		      session->gpsdata.dev.path,
+		      sizeof(session->packet.outbuffer));
+	(void)strlcat((char *)session->packet.outbuffer, "\"}", 
+		      sizeof(session->packet.outbuffer));
+    }
+
+    gpsd_report (LOG_PROG, 
+		 "JSON, passing through %s\n", 
+		 (char *)session->packet.outbuffer);
+    /*@-nullpass@*/
+    return PASSTHROUGH_IS;
+}
+
+/* *INDENT-OFF* */
+const struct gps_type_t json_passthrough = {
+    .type_name      = "JSON slave driver",	/* full name of type */
+    .packet_type    = JSON_PACKET,	/* associated lexer packet type */
+    .flags	    = DRIVER_NOFLAGS,	/* no flags set */
+    .trigger	    = NULL,		/* it's the default */
+    .channels       = 0,		/* not used */
+    .probe_detect   = NULL,		/* no probe */
+    .get_packet     = generic_get,	/* use generic packet getter */
+    .parse_packet   = json_pass_packet,	/* how to interpret a packet */
+    .rtcm_writer    = NULL,		/* write RTCM data straight */
+    .event_hook     = NULL,		/* lifetime event handler */
+#ifdef RECONFIGURE_ENABLE
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .min_cycle      = 1,		/* not relevant, no rate switch */
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
+    .control_send   = NULL,		/* how to send control strings */
+#endif /* CONTROLSEND_ENABLE */
+#ifdef NTPSHM_ENABLE
+    .ntp_offset     = NULL,		/* no method for NTP fudge factor */
+#endif /* NTPSHM_ ENABLE */
+};
+/* *INDENT-ON* */
+
+#endif /* PASSTHROUGH_ENABLE */
+
 extern const struct gps_type_t garmin_usb_binary, garmin_ser_binary;
 extern const struct gps_type_t geostar_binary;
 extern const struct gps_type_t tsip_binary, oncore_binary;
@@ -1281,6 +1363,11 @@ static const struct gps_type_t *gpsd_driver_array[] = {
 #ifdef GARMINTXT_ENABLE
     &garmintxt,
 #endif /* GARMINTXT_ENABLE */
+
+#ifdef PASSTHROUGH_ENABLE
+    &json_passthrough,
+#endif /* PASSTHROUGH_ENABLE */
+
     NULL,
 };
 
