@@ -1,3 +1,4 @@
+/* gps.h -- interface of the libgps library */
 /*
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
@@ -5,12 +6,6 @@
 #ifndef _GPSD_GPS_H_
 #define _GPSD_GPS_H_
 
-/* gps.h -- interface of the libgps library */
-
-#ifdef _WIN32
-#define strtok_r(s,d,p) strtok_s(s,d,p)
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -264,7 +259,7 @@ struct rtcm3_rtk_hdr {		/* header data from 1001, 1002, 1003, 1004 */
 
 struct rtcm3_basic_rtk {
     unsigned char indicator;	/* Indicator */
-    unsigned char channel;	/* Satellite Frequency Channel Number 
+    short channel;		/* Satellite Frequency Channel Number 
 				   (GLONASS only) */
     double pseudorange;		/* Pseudorange */
     double rangediff;		/* PhaseRange – Pseudorange in meters */
@@ -273,7 +268,7 @@ struct rtcm3_basic_rtk {
 
 struct rtcm3_extended_rtk {
     unsigned char indicator;	/* Indicator */
-    unsigned char channel;	/* Satellite Frequency Channel Number 
+    short channel;		/* Satellite Frequency Channel Number 
 				   (GLONASS only) */
     double pseudorange;		/* Pseudorange */
     double rangediff;		/* PhaseRange – L1 Pseudorange */
@@ -404,9 +399,9 @@ struct rtcm3_t {
 	    unsigned char leapsecs;	/* Leap Seconds, GPS-UTC */
 	    unsigned char ncount;	/* Count of announcements to follow */
 	    struct {
-		unsigned short id;
+		unsigned short id;		/* message type ID */
 		bool sync;
-		unsigned short interval;
+		unsigned short interval;	/* interval in 0.1sec units */
 	    } announcements[RTCM3_MAX_ANNOUNCEMENTS];
 	} rtcm3_1013;
 	/* 1014-1017 were added in the 3.1 version */
@@ -510,10 +505,23 @@ struct rtcm3_t {
 	    size_t unicode_units;	/* # Unicode units in text */
 	    unsigned char text[128];
 	} rtcm3_1029;
+	struct rtcm3_1033_t {
+	    unsigned int station_id;			/* Reference Station ID */
+	    char descriptor[RTCM3_MAX_DESCRIPTOR+1];	/* Description string */
+	    unsigned char setup_id;
+	    char serial[RTCM3_MAX_DESCRIPTOR+1];	/* Serial # string */
+	    char receiver[RTCM3_MAX_DESCRIPTOR+1];	/* Receiver string */
+	    char firmware[RTCM3_MAX_DESCRIPTOR+1];	/* Firmware string */
+	} rtcm3_1033;
 	char data[1024];		/* Max RTCM3 msg length is 1023 bytes */
     } rtcmtypes;
 };
 
+/* RTCM3 scaling constants */
+#define GPS_AMBIGUITY_MODULUS		299792.458	/* 1004, DF014*/
+#define GLONASS_AMBIGUITY_MODULUS	599584.916	/* 1012, DF044 */
+#define MESSAGE_INTERVAL_UNITS		0.1		/* 1013, DF047 */
+
 /*
  * Raw IS_GPS subframe data
  */
@@ -828,6 +836,32 @@ typedef /*@unsignedintegraltype@*/ unsigned long long gps_mask_t;
  */
 #define AIS_AUXILIARY_MMSI(n)	((n) / 10000000 == 98)
 
+/* N/A values and scaling constant for 25/24 bit lon/lat pairs */
+#define AIS_LON3_NOT_AVAILABLE	181000
+#define AIS_LAT3_NOT_AVAILABLE	91000
+#define AIS_LATLON3_SCALE	60000.0
+
+/* N/A values and scaling constant for 28/27 bit lon/lat pairs */
+#define AIS_LON4_NOT_AVAILABLE	1810000
+#define AIS_LAT4_NOT_AVAILABLE	910000
+#define AIS_LATLON4_SCALE	600000.0
+
+struct route_info {
+    unsigned int linkage;	/* Message Linkage ID */
+    unsigned int sender;	/* Sender Class */
+    unsigned int rtype;		/* Route Type */
+    unsigned int month;		/* Start month */
+    unsigned int day;		/* Start day */
+    unsigned int hour;		/* Start hour */
+    unsigned int minute;	/* Start minute */
+    unsigned int duration;	/* Duration */
+    int waycount;		/* Waypoint count */
+    struct waypoint_t {
+	signed int lon;		/* Longitude */
+	signed int lat;		/* Latitude */
+    } waypoints[16];
+};
+
 struct ais_t
 {
     unsigned int	type;		/* message type */
@@ -890,7 +924,7 @@ struct ais_t
 	struct {
 	    unsigned int ais_version;	/* AIS version level */
 	    unsigned int imo;		/* IMO identification */
-	    char callsign[8];		/* callsign */ 
+	    char callsign[7+1];		/* callsign */ 
 #define AIS_SHIPNAME_MAXLEN	20
 	    char shipname[AIS_SHIPNAME_MAXLEN+1];	/* vessel name */
 	    unsigned int shiptype;	/* ship type code */
@@ -904,7 +938,7 @@ struct ais_t
 	    unsigned int hour;		/* UTC hour */
 	    unsigned int minute;	/* UTC minute */
 	    unsigned int draught;	/* draft in meters */
-	    char destination[21];	/* ship destination */
+	    char destination[20+1];	/* ship destination */
 	    unsigned int dte;		/* data terminal enable */
 	    //unsigned int spare;	spare bits */
 	} type5;
@@ -918,7 +952,132 @@ struct ais_t
 	    unsigned int fid;           /* Functional ID */
 #define AIS_TYPE6_BINARY_MAX	920	/* 920 bits */
 	    size_t bitcount;		/* bit count of the data */
-	    char bitdata[(AIS_TYPE6_BINARY_MAX + 7) / 8];
+	    union {
+		char bitdata[(AIS_TYPE6_BINARY_MAX + 7) / 8];
+		/* IMO236 - Dangerous Cargo Indication */
+		struct {
+		    char lastport[5+1];		/* Last Port Of Call */
+		    unsigned int lmonth;	/* ETA month */
+		    unsigned int lday;		/* ETA day */
+		    unsigned int lhour;		/* ETA hour */
+		    unsigned int lminute;	/* ETA minute */
+		    char nextport[5+1];		/* Next Port Of Call */
+		    unsigned int nmonth;	/* ETA month */
+		    unsigned int nday;		/* ETA day */
+		    unsigned int nhour;		/* ETA hour */
+		    unsigned int nminute;	/* ETA minute */
+		    char dangerous[20+1];	/* Main Dangerous Good */
+		    char imdcat[4+1];		/* IMD Category */
+		    unsigned int unid;		/* UN Number */
+		    unsigned int amount;	/* Amount of Cargo */
+		    unsigned int unit;		/* Unit of Quantity */
+		} dac1fid12;
+		/* IMO236 - Extended Ship Static and Voyage Related Data */
+		struct {
+		    unsigned int airdraught;	/* Air Draught */
+		} dac1fid15;
+		/* IMO236 - Number of Persons on board */
+		struct {
+		    unsigned persons;	/* number of persons */
+		} dac1fid16;
+		/* IMO289 - Clearance Time To Enter Port */
+		struct {
+		    unsigned int linkage;	/* Message Linkage ID */
+		    unsigned int month;	/* Month (UTC) */
+		    unsigned int day;	/* Day (UTC) */
+		    unsigned int hour;	/* Hour (UTC) */
+		    unsigned int minute;	/* Minute (UTC) */
+		    char portname[20+1];	/* Name of Port & Berth */
+		    char destination[5+1];	/* Destination */
+		    signed int lon;	/* Longitude */
+		    signed int lat;	/* Latitude */
+		} dac1fid18;
+		/* IMO289 - Berthing Data (addressed) */
+		struct {
+		    unsigned int linkage;	/* Message Linkage ID */
+		    unsigned int berth_length;	/* Berth length */
+		    unsigned int berth_depth;	/* Berth Water Depth */
+		    unsigned int position;	/* Mooring Position */
+		    unsigned int month;	/* Month (UTC) */
+		    unsigned int day;	/* Day (UTC) */
+		    unsigned int hour;	/* Hour (UTC) */
+		    unsigned int minute;	/* Minute (UTC) */
+		    unsigned int availability;	/* Services Availability */
+		    unsigned int agent;	/* Agent */
+		    unsigned int fuel;	/* Bunker/fuel */
+		    unsigned int chandler;	/* Chandler */
+		    unsigned int stevedore;	/* Stevedore */
+		    unsigned int electrical;	/* Electrical */
+		    unsigned int water;	/* Potable water */
+		    unsigned int customs;	/* Customs house */
+		    unsigned int cartage;	/* Cartage */
+		    unsigned int crane;	/* Crane(s) */
+		    unsigned int lift;	/* Lift(s) */
+		    unsigned int medical;	/* Medical facilities */
+		    unsigned int navrepair;	/* Navigation repair */
+		    unsigned int provisions;	/* Provisions */
+		    unsigned int shiprepair;	/* Ship repair */
+		    unsigned int surveyor;	/* Surveyor */
+		    unsigned int steam;	/* Steam */
+		    unsigned int tugs;	/* Tugs */
+		    unsigned int solidwaste;	/* Waste disposal (solid) */
+		    unsigned int liquidwaste;	/* Waste disposal (liquid) */
+		    unsigned int hazardouswaste;	/* Waste disposal (hazardous) */
+		    unsigned int ballast;	/* Reserved ballast exchange */
+		    unsigned int additional;	/* Additional services */
+		    unsigned int regional1;	/* Regional reserved 1 */
+		    unsigned int regional2;	/* Regional reserved 2 */
+		    unsigned int future1;	/* Reserved for future */
+		    unsigned int future2;	/* Reserved for future */
+		    char berth_name[20+1];	/* Name of Berth */
+		    signed int berth_lon;	/* Longitude */
+		    signed int berth_lat;	/* Latitude */
+		} dac1fid20;
+		/* IMO289 - Dangerous Cargo Indication */
+		struct {
+		    unsigned int unit;	/* Unit of Quantity */
+		    unsigned int amount;	/* Amount of Cargo */
+		    int ncargos;
+		    struct cargo_t {
+			unsigned int code;	/* Cargo code */
+			unsigned int subtype;	/* Cargo subtype */
+		    } cargos[28];
+		} dac1fid25;
+		/* IMO289 - Route info (addressed) */
+		struct route_info dac1fid28;
+		/* IMO289 - Text message (addressed) */
+		struct {
+		    unsigned int linkage;
+#define AIS_DAC1FID30_TEXT_MAX	154	/* 920 bits of six-bit, plus NUL */
+		    char text[AIS_DAC1FID30_TEXT_MAX];
+		} dac1fid30;
+		/* IMO289 & IMO236 - Tidal Window */
+		struct {
+		    unsigned int type;	/* Message Type */
+		    unsigned int repeat;	/* Repeat Indicator */
+		    unsigned int mmsi;	/* Source MMSI */
+		    unsigned int seqno;	/* Sequence Number */
+		    unsigned int dest_mmsi;	/* Destination MMSI */
+		    signed int retransmit;	/* Retransmit flag */
+		    unsigned int dac;	/* DAC */
+		    unsigned int fid;	/* FID */
+		    unsigned int month;	/* Month */
+		    unsigned int day;	/* Day */
+		    signed int ntidals;
+		    struct tidal_t {
+			signed int lon;	/* Longitude */
+			signed int lat;	/* Latitude */
+			unsigned int from_hour;	/* From UTC Hour */
+			unsigned int from_min;	/* From UTC Minute */
+			unsigned int to_hour;	/* To UTC Hour */
+			unsigned int to_min;	/* To UTC Minute */
+#define DAC1FID32_CDIR_NOT_AVAILABLE		360
+			unsigned int cdir;	/* Current Dir. Predicted */
+#define DAC1FID32_CSPEED_NOT_AVAILABLE		127
+			unsigned int cspeed;	/* Current Speed Predicted */
+		    } tidals[3];
+		} dac1fid32;
+	    };
 	} type6;
 	/* Type 7 - Binary Acknowledge */
 	struct {
@@ -937,14 +1096,81 @@ struct ais_t
 	    size_t bitcount;		/* bit count of the data */
 	    union {
 		char bitdata[(AIS_TYPE8_BINARY_MAX + 7) / 8];
-#ifdef __UNUSED__
+		/* IMO236 - Fairway Closed */
 		struct {
+		    char reason[20+1];	/* Reason For Closing */
+		    char closefrom[20+1];	/* Location Of Closing From */
+		    char closeto[20+1];	/* Location of Closing To */
+		    unsigned int radius;	/* Radius extension */
+#define AIS_DAC1FID13_RADIUS_NOT_AVAILABLE 10001
+		    unsigned int extunit;	/* Unit of extension */
+#define AIS_DAC1FID13_EXTUNIT_NOT_AVAILABLE 0
+		    unsigned int fday;	/* From day (UTC) */
+		    unsigned int fmonth;	/* From month (UTC) */
+		    unsigned int fhour;	/* From hour (UTC) */
+		    unsigned int fminute;	/* From minute (UTC) */
+		    unsigned int tday;	/* To day (UTC) */
+		    unsigned int tmonth;	/* To month (UTC) */
+		    unsigned int thour;	/* To hour (UTC) */
+		    unsigned int tminute;	/* To minute (UTC) */
+		} dac1fid13;
+	        /* IMO236 - Extended ship and voyage data */
+		struct {
+		    unsigned int airdraught;	/* Air Draught */
+		} dac1fid15;
+		/* IMO289 - VTS-generated/Synthetic Targets */
+		struct {
+		    signed int ntargets;
+		    struct target_t {
+#define DAC1FID17_IDTYPE_MMSI		0
+#define DAC1FID17_IDTYPE_IMO		1
+#define DAC1FID17_IDTYPE_CALLSIGN	2
+#define DAC1FID17_IDTYPE_OTHER		3
+			unsigned int idtype;	/* Identifier type */
+			union target_id {	/* Target identifier */
+			    unsigned int mmsi;
+			    unsigned int imo;
+#define DAC1FID17_ID_LENGTH		7
+			    char callsign[DAC1FID17_ID_LENGTH+1];
+			    char other[DAC1FID17_ID_LENGTH+1];
+			} id;
+			signed int lat;		/* Latitude */
+			signed int lon;		/* Longitude */
+#define DAC1FID17_COURSE_NOT_AVAILABLE		360
+			unsigned int course;	/* Course Over Ground */
+			unsigned int second;	/* Time Stamp */
+#define DAC1FID17_SPEED_NOT_AVAILABLE		255
+			unsigned int speed;	/* Speed Over Ground */
+		    } targets[4];
+		} dac1fid17;
+		/* IMO 289 - Marine Traffic Signal */
+		struct {
+		    unsigned int linkage;	/* Message Linkage ID */
+		    char station[20+1];		/* Name of Signal Station */
+		    signed int lon;		/* Longitude */
+		    signed int lat;		/* Latitude */
+		    unsigned int status;	/* Status of Signal */
+		    unsigned int signal;	/* Signal In Service */
+		    unsigned int hour;		/* UTC hour */
+		    unsigned int minute;	/* UTC minute */
+		    unsigned int nextsignal;	/* Expected Next Signal */
+		} dac1fid19;
+		/* IMO289 - Route info (broadcast) */
+		struct route_info dac1fid27;
+		/* IMO289 - Text message (broadcast) */
+		struct {
+		    unsigned int linkage;
+#define AIS_DAC1FID29_TEXT_MAX	162	/* 920 bits of six-bit, plus NUL */
+		    char text[AIS_DAC1FID29_TEXT_MAX];
+		} dac1fid29;
+		/* IMO236 & IMO289 - Meteorological-Hydrological data */
+		struct {
+		    bool accuracy;	/* position accuracy, <10m if true */
 #define DAC1FID31_LATLON_SCALE	1000
 		    int lon;		/* longitude in minutes * .001 */
 #define DAC1FID31_LON_NOT_AVAILABLE	(181*60*DAC1FID31_LATLON_SCALE)
 		    int lat;		/* longitude in minutes * .001 */
 #define DAC1FID31_LAT_NOT_AVAILABLE	(91*60*DAC1FID31_LATLON_SCALE)
-		    bool accuracy;	/* position accuracy, <10m if true */
 		    unsigned int day;		/* UTC day */
 		    unsigned int hour;		/* UTC hour */
 		    unsigned int minute;	/* UTC minute */
@@ -955,8 +1181,8 @@ struct ais_t
 		    unsigned int wdir;		/* wind direction */
 		    unsigned int wgustdir;	/* wind gust direction */
 #define DAC1FID31_DIR_NOT_AVAILABLE		360
-		    int temperature;		/* temperature, units 0.1C */
-#define DAC1FID31_AIRTEMP_NOT_AVAILABLE		-1024
+		    int airtemp;		/* temperature, units 0.1C */
+#define DAC1FID31_AIRTEMP_NOT_AVAILABLE		-1084
 		    unsigned int humidity;	/* relative humidity, % */
 #define DAC1FID31_HUMIDITY_NOT_AVAILABLE	101
 		    int dewpoint;		/* dew point, units 0.1C */
@@ -965,13 +1191,42 @@ struct ais_t
 #define DAC1FID31_PRESSURE_NOT_AVAILABLE	511
 #define DAC1FID31_PRESSURE_HIGH			402
 		    unsigned int pressuretend;	/* tendency */
-		    bool visgreater;		/* vis. > than following */
+#define DAC1FID31_PRESSURETREND_NOT_AVAILABLE	3
+		    bool visgreater;            /* visibility greater than */
 		    unsigned int visibility;	/* units 0.1 nautical miles */
 #define DAC1FID31_VISIBILITY_NOT_AVAILABLE	127
-		    unsigned int waterlevel;	/* decimeters + 100 */
-#define DAC1FID31_WATERLEVEL_NOT_AVAILABLE	4001
+		    int waterlevel;		/* decimeters or cm */
+#define DAC1FID11_WATERLEVEL_NOT_AVAILABLE	4001
+#define DAC1FID31_WATERLEVEL_NOT_AVAILABLE	40001
+		    unsigned int leveltrend;	/* water level trend code */
+#define DAC1FID31_LEVELTREND_NOT_AVAILABLE	3
+		    unsigned int cspeed;	/* current speed in deciknots */
+#define DAC1FID31_CSPEED_NOT_AVAILABLE		255
+		    unsigned int cdir;		/* current dir., degrees */
+		    unsigned int cspeed2;	/* current speed in deciknots */
+		    unsigned int cdir2;		/* current dir., degrees */
+		    unsigned int cdepth2;	/* measurement depth, 0.1m */
+#define DAC1FID31_CDEPTH_NOT_AVAILABLE		301
+		    unsigned int cspeed3;	/* current speed in deciknots */
+		    unsigned int cdir3;		/* current dir., degrees */
+		    unsigned int cdepth3;	/* measurement depth, 0.1m */
+		    unsigned int waveheight;	/* in decimeters */
+#define DAC1FID31_HEIGHT_NOT_AVAILABLE		31
+		    unsigned int waveperiod;	/* in seconds */
+#define DAC1FID31_PERIOD_NOT_AVAILABLE		63
+		    unsigned int wavedir;	/* direction in degrees */
+		    unsigned int swellheight;	/* in decimeters */
+		    unsigned int swellperiod;	/* in seconds */
+		    unsigned int swelldir;	/* direction in degrees */
+		    unsigned int seastate;	/* Beaufort scale, 0-12 */
+#define DAC1FID31_SEASTATE_NOT_AVAILABLE	15
+		    int watertemp;		/* units 0.1deg Celsius */
+#define DAC1FID31_PRECIPTYPE_NOT_AVAILABLE	7
+		    unsigned int preciptype;	/* 0-7, enumerated */
+		    unsigned int salinity;	/* units of 0.1% */
+#define DAC1FID31_SALINITY_NOT_AVAILABLE	510
+		    bool ice;			/* is there sea ice? */
 		} dac1fid31;
-#endif /* __UNUSED_ */
 	    };
 	} type8;
 	/* Type 9 - Standard SAR Aircraft Position Report */
@@ -1213,6 +1468,22 @@ struct ais_t
 	    char bitdata[(AIS_TYPE26_BINARY_MAX + 7) / 8];
 	    unsigned int radio;		/* radio status bits */
 	} type26;
+	/* Type 27 - Long Range AIS Broadcast message */
+	struct {
+	    bool accuracy;		/* position accuracy */
+	    bool raim;			/* RAIM flag */
+	    unsigned int status;	/* navigation status */
+#define AIS_LONGRANGE_LATLON_SCALE	600.0
+	    int lon;			/* longitude */
+#define AIS_LONGRANGE_LON_NOT_AVAILABLE	0x1a838
+	    int lat;			/* latitude */
+#define AIS_LONGRANGE_LAT_NOT_AVAILABLE	0xd548
+	    unsigned int speed;		/* speed over ground in deciknots */
+#define AIS_LONGRANGE_SPEED_NOT_AVAILABLE 63
+	    unsigned int course;	/* course over ground */
+#define AIS_LONGRANGE_COURSE_NOT_AVAILABLE 511
+	    bool gnss;			/* are we reporting GNSS position? */
+	} type27;
     };
 };
 
@@ -1268,6 +1539,7 @@ struct version_t {
     char release[64];			/* external version */
     char rev[64];			/* internal revision ID */
     int proto_major, proto_minor;	/* API major and minor versions */
+    char remote[GPS_PATH_MAX];		/* could be from a remote device */
 };
 
 struct devconfig_t {
@@ -1295,6 +1567,7 @@ struct policy_t {
     bool timing;			/* requesting timing info */
     int loglevel;			/* requested log level of messages */
     char devpath[GPS_PATH_MAX];		/* specific device to watch */
+    char remote[GPS_PATH_MAX];		/* ...if this was passthrough */
 };
 
 /* 
@@ -1324,37 +1597,38 @@ typedef int socket_t;
 
 struct gps_data_t {
     gps_mask_t set;	/* has field been set since this was last cleared? */
-#define ONLINE_SET	(1u<<1)
-#define TIME_SET	(1u<<2)
-#define TIMERR_SET	(1u<<3)
-#define LATLON_SET	(1u<<4)
-#define ALTITUDE_SET	(1u<<5)
-#define SPEED_SET	(1u<<6)
-#define TRACK_SET	(1u<<7)
-#define CLIMB_SET	(1u<<8)
-#define STATUS_SET	(1u<<9)
-#define MODE_SET	(1u<<10)
-#define DOP_SET  	(1u<<11)
-#define HERR_SET	(1u<<12)
-#define VERR_SET	(1u<<13)
-#define ATTITUDE_SET	(1u<<14)
-#define SATELLITE_SET	(1u<<15)
-#define SPEEDERR_SET	(1u<<16)
-#define TRACKERR_SET	(1u<<17)
-#define CLIMBERR_SET	(1u<<18)
-#define DEVICE_SET	(1u<<19)
-#define DEVICELIST_SET	(1u<<20)
-#define DEVICEID_SET	(1u<<21)
-#define RTCM2_SET	(1u<<22)
-#define RTCM3_SET	(1u<<23)
-#define AIS_SET 	(1u<<24)
-#define PACKET_SET	(1u<<25)
-#define SUBFRAME_SET	(1u<<26)
-#define GST_SET 	(1u<<27)
-#define VERSION_SET	(1u<<28)
-#define POLICY_SET	(1u<<29)
-#define LOGMESSAGE_SET	(1u<<30)
-#define ERROR_SET	(1u<<31)
+#define ONLINE_SET	(1llu<<1)
+#define TIME_SET	(1llu<<2)
+#define TIMERR_SET	(1llu<<3)
+#define LATLON_SET	(1llu<<4)
+#define ALTITUDE_SET	(1llu<<5)
+#define SPEED_SET	(1llu<<6)
+#define TRACK_SET	(1llu<<7)
+#define CLIMB_SET	(1llu<<8)
+#define STATUS_SET	(1llu<<9)
+#define MODE_SET	(1llu<<10)
+#define DOP_SET  	(1llu<<11)
+#define HERR_SET	(1llu<<12)
+#define VERR_SET	(1llu<<13)
+#define ATTITUDE_SET	(1llu<<14)
+#define SATELLITE_SET	(1llu<<15)
+#define SPEEDERR_SET	(1llu<<16)
+#define TRACKERR_SET	(1llu<<17)
+#define CLIMBERR_SET	(1llu<<18)
+#define DEVICE_SET	(1llu<<19)
+#define DEVICELIST_SET	(1llu<<20)
+#define DEVICEID_SET	(1llu<<21)
+#define RTCM2_SET	(1llu<<22)
+#define RTCM3_SET	(1llu<<23)
+#define AIS_SET 	(1llu<<24)
+#define PACKET_SET	(1llu<<25)
+#define SUBFRAME_SET	(1llu<<26)
+#define GST_SET 	(1llu<<27)
+#define VERSION_SET	(1llu<<28)
+#define POLICY_SET	(1llu<<29)
+#define LOGMESSAGE_SET	(1llu<<30)
+#define ERROR_SET	(1llu<<31)
+#define SET_HIGH_BIT	31
     timestamp_t online;		/* NZ if GPS is on line, 0 if not.
 				 *
 				 * Note: gpsd clears this time when sentences
@@ -1400,10 +1674,11 @@ struct gps_data_t {
 
     struct policy_t policy;	/* our listening policy */
 
+    /* should be moved to privdata sometday */
     char tag[MAXTAGLEN+1];	/* tag of last sentence processed */
 
     /* pack things never reported together to reduce structure size */ 
-#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|VERSION_SET|DEVICELIST_SET|LOGMESSAGE_SET|ERROR_SET|GST_SET)
+#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|ATTITUDE_SET|VERSION_SET|DEVICELIST_SET|LOGMESSAGE_SET|ERROR_SET|GST_SET|VERSION_SET)
     union {
 	/* unusual forms of sensor data that might come up the pipe */ 
 	struct rtcm2_t	rtcm2;
@@ -1432,17 +1707,26 @@ extern int gps_open(/*@null@*/const char *, /*@null@*/const char *,
 extern int gps_close(struct gps_data_t *);
 extern int gps_send(struct gps_data_t *, const char *, ... );
 extern int gps_read(/*@out@*/struct gps_data_t *);
-extern bool gps_waiting(struct gps_data_t *, int);
+extern int gps_unpack(char *, struct gps_data_t *);
+extern bool gps_waiting(const struct gps_data_t *, int);
 extern int gps_stream(struct gps_data_t *, unsigned int, /*@null@*/void *);
-extern const char /*@observer@*/ *gps_data(struct gps_data_t *);
+extern const char /*@observer@*/ *gps_data(const struct gps_data_t *);
 extern const char /*@observer@*/ *gps_errstr(const int);
 
-/* this only needs to be visible for the unit tests */
-extern int gps_unpack(char *, struct gps_data_t *);
+extern int gps_sock_open(/*@null@*/const char *, /*@null@*/const char *, 
+		      /*@out@*/struct gps_data_t *);
+extern int gps_sock_read(/*@out@*/struct gps_data_t *);
+extern int gps_sock_close(struct gps_data_t *);
+extern int gps_shm_open(/*@out@*/struct gps_data_t *);
+extern int gps_shm_read(struct gps_data_t *);
+extern void gps_shm_close(struct gps_data_t *);
+
+extern void libgps_trace(int errlevel, const char *, ...);
 
 /* dependencies on struct gpsdata_t end hrere */
 
 extern void gps_clear_fix(/*@ out @*/struct gps_fix_t *);
+extern void gps_clear_dop( /*@out@*/ struct dop_t *);
 extern void gps_merge_fix(/*@ out @*/struct gps_fix_t *,
 			  gps_mask_t,
 			  /*@ in @*/struct gps_fix_t *);
@@ -1487,10 +1771,31 @@ extern double wgs84_separation(double, double);
 #define NL_NOSOCK	-4	/* can't create socket */
 #define NL_NOSOCKOPT	-5	/* error SETSOCKOPT SO_REUSEADDR */
 #define NL_NOCONNECT	-6	/* can't connect to host/socket pair */
+#define SHM_NOSHARED	-7	/* shared-memory segment not available */
+#define SHM_NOATTACH	-8	/* shared-memory attach failed */
 
 #define DEFAULT_GPSD_PORT	"2947"	/* IANA assignment */
 #define DEFAULT_RTCM_PORT	"2101"	/* IANA assignment */
 
+/* special host values for non-socket exports */ 
+#define GPSD_SHARED_MEMORY	"shared memory"
+
+/*
+ * Platform-specific declarations
+ */
+
+#ifdef _WIN32
+#define strtok_r(s,d,p) strtok_s(s,d,p)
+#endif
+
+/* Some libc's don't have strlcat/strlcpy. Local copies are provided */
+#ifndef HAVE_STRLCAT
+size_t strlcat(/*@out@*/char *dst, /*@in@*/const char *src, size_t size);
+#endif
+#ifndef HAVE_STRLCPY
+size_t strlcpy(/*@out@*/char *dst, /*@in@*/const char *src, size_t size);
+#endif
+
 #ifdef __cplusplus
 }  /* End of the 'extern "C"' block */
 #endif
