@@ -26,5 +26,10 @@ extern /*@observer@*/ char *deg_to_str( enum deg_str_type type,  double f);
 extern void gpsd_source_spec(/*@null@*/const char *fromstring, 
 			     /*@out@*/struct fixsource_t *source);
 
+char *maidenhead(double n,double e);
+
+/* this needs to match JSON_DATE_MAX in gpsd.h */
+#define CLIENT_DATE_MAX	24
+
 #endif /* _GPSDCLIENT_H_ */
 /* gpsdclient.h ends here */
