@@ -17,6 +17,8 @@
 static int verbose = 0;
 static bool scaled = true;
 static bool json = true;
+static unsigned int ntypes = 0;
+static unsigned int typelist[32];
 
 /**************************************************************************
  *
@@ -36,7 +38,7 @@ void gpsd_report(int errlevel, const char *fmt, ...)
 	(void)vsnprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), fmt,
 			ap);
 	va_end(ap);
-	(void)fputs(buf, stdout);
+	(void)fputs(buf, stderr);
     }
 }
 
@@ -358,6 +360,29 @@ static void aivdm_csv_dump(struct ais_t *ais, char *buf, size_t buflen)
 }
 #endif
 
+static bool filter(gps_mask_t changed, struct gps_device_t *session)
+/* say whether a given message should be visible */
+{
+    if (ntypes == 0)
+	return true;
+    else {
+	unsigned int i, t;
+
+	if ((changed & AIS_SET)!=0)
+	    t = session->gpsdata.ais.type;
+	else if ((changed & RTCM2_SET)!=0)
+	    t = session->gpsdata.rtcm2.type;
+	else if ((changed & RTCM3_SET)!=0)
+	    t = session->gpsdata.rtcm3.type;
+	else
+	    return true;
+	for (i = 0; i < ntypes; i++)
+	    if (t == typelist[i])
+		return true;
+    }
+    return false;
+}
+
 /*@ -compdestroy -compdef -usedef -uniondef @*/
 static void decode(FILE *fpin, FILE*fpout)
 /* sensor data on fpin to dump format on fpout */
@@ -367,10 +392,13 @@ static void decode(FILE *fpin, FILE*fpout)
     struct policy_t policy;
     char buf[GPS_JSON_RESPONSE_MAX * 4];
 
+    //This looks like a good idea, but it breaks regression tests
+    //(void)strlcpy(session.gpsdata.dev.path, "stdin", sizeof(session.gpsdata.dev.path));
     memset(&policy, '\0', sizeof(policy));
     policy.json = json;
 
     gps_context_init(&context);
+    gpsd_time_init(&context, time(NULL));
     context.readonly = true;
     gpsd_init(&session, &context, NULL);
     gpsd_clear(&session);
@@ -381,20 +409,28 @@ static void decode(FILE *fpin, FILE*fpout)
     {
 	gps_mask_t changed = gpsd_poll(&session);
 
-	if (changed == ERROR_IS || changed == NODATA_IS)
+	if (changed == ERROR_SET || changed == NODATA_IS)
 	    break;
+	if (session.packet.type == COMMENT_PACKET)
+	    gpsd_set_century(&session);
 	if (verbose >= 1 && TEXTUAL_PACKET_TYPE(session.packet.type))
 	    (void)fputs((char *)session.packet.outbuffer, fpout);
-	if ((changed & (REPORT_IS|SUBFRAME_IS|AIS_IS|RTCM2_IS|RTCM3_IS)) == 0)
+	if ((changed & (REPORT_IS|SUBFRAME_SET|AIS_SET|RTCM2_SET|RTCM3_SET|PASSTHROUGH_IS)) == 0)
+	    continue;
+	if (!filter(changed, &session))
 	    continue;
 	else if (json) {
-	    json_data_report(changed, 
-			     &session.gpsdata, &policy, 
-			     buf, sizeof(buf));
+	    if ((changed & PASSTHROUGH_IS) != 0) {
+		(void)fputs((char *)session.packet.outbuffer, fpout);
+		(void)fputs("\n", fpout);
+	    } else
+		json_data_report(changed, 
+				 &session.gpsdata, &policy, 
+				 buf, sizeof(buf));
 	    (void)fputs(buf, fpout);	
 #ifdef AIVDM_ENABLE
 	} else if (session.packet.type == AIVDM_PACKET) {
-	    if ((changed & AIS_IS)!=0) {
+	    if ((changed & AIS_SET)!=0) {
 		aivdm_csv_dump(&session.gpsdata.ais, buf, sizeof(buf));
 		(void)fputs(buf, fpout);
 	    }
@@ -441,7 +477,7 @@ int main(int argc, char **argv)
     enum
     { doencode, dodecode } mode = dodecode;
 
-    while ((c = getopt(argc, argv, "cdejpuvVD:")) != EOF) {
+    while ((c = getopt(argc, argv, "cdejpt:uvVD:")) != EOF) {
 	switch (c) {
 	case 'c':
 	    json = false;
@@ -459,6 +495,18 @@ int main(int argc, char **argv)
 	    json = true;
 	    break;
 
+	case 't':
+	    /*@-nullpass@*/
+	    typelist[ntypes++] = (unsigned int)atoi(strtok(optarg, ","));
+	    for(;;) {
+		char *next = strtok(NULL, ",");
+		if (next == NULL)
+		    break;
+		typelist[ntypes++] = (unsigned int)atoi(next);
+	    }
+	    /*@+nullpass@*/
+	    break;
+
 	case 'u':
 	    scaled = false;
 	    break;
@@ -469,8 +517,7 @@ int main(int argc, char **argv)
 		
 	case 'D':
 	    verbose = atoi(optarg);
-	    gpsd_hexdump_level = verbose;
-#ifdef CLIENTDEBUG_ENABLE
+#if defined(CLIENTDEBUG_ENABLE) && defined(SOCKET_EXPORT_ENABLE)
 	    json_enable_debug(verbose - 2, stderr);
 #endif
 	    break;
@@ -485,8 +532,8 @@ int main(int argc, char **argv)
 	    exit(1);
 	}
     }
-    argc -= optind;
-    argv += optind;
+    //argc -= optind;
+    //argv += optind;
 
     if (mode == doencode)
 	encode(stdin, stdout);
