@@ -24,11 +24,6 @@
 #include <unistd.h>
 #endif /* S_SPLINT_S */
 
-/* Cygwin has only _timezone and not timezone unless the following is set */
-#if defined(__CYGWIN__)
-#define timezonevar
-#endif /* defined(__CYGWIN__) */
-
 #include "gpsd_config.h"
 #ifdef HAVE_BLUEZ
 #include <bluetooth/bluetooth.h>
@@ -38,11 +33,6 @@
 #include "gpsmon.h"
 #include "revision.h"
 
-
-#ifdef S_SPLINT_S
-extern struct tm *localtime_r(const time_t *, /*@out@*/ struct tm *tp);
-#endif /* S_SPLINT_S */
-
 #define BUFLEN		2048
 
 /* external capability tables */
@@ -57,13 +47,23 @@ WINDOW *devicewin;
 
 /* These are private */
 static struct gps_context_t context;
-static int controlfd = -1;
 static bool serial, curses_active;
-static int debuglevel = 0;
 static WINDOW *statwin, *cmdwin;
 /*@null@*/ static WINDOW *packetwin;
 /*@null@*/ static FILE *logfile;
 static char *type_name;
+
+/* no methods, it's all device window */
+extern const struct gps_type_t json_passthrough;
+const struct monitor_object_t json_mmt = {
+    .initialize = NULL,
+    .update = NULL,
+    .command = NULL,
+    .wrap = NULL,
+    .min_y = 0, .min_x = 80,	/* no need for a device window */
+    .driver = &json_passthrough,
+};
+
 /*@ -nullassign @*/
 static const struct monitor_object_t *monitor_objects[] = {
 #ifdef NMEA_ENABLE
@@ -102,6 +102,9 @@ static const struct monitor_object_t *monitor_objects[] = {
 #ifdef TNT_ENABLE
     &tnt_mmt,
 #endif /* TNT_ENABLE */
+#ifdef PASSTHROUGH_ENABLE
+    &json_mmt,
+#endif /* PASSTHROUGH_ENABLE */
     NULL,
 };
 
@@ -189,7 +192,7 @@ void gpsd_report(int errlevel, const char *fmt, ...)
 
     (void)strlcpy(buf, "gpsd:", BUFSIZ);
     (void)strncat(buf, err_str, BUFSIZ - strlen(buf) );
-    if (errlevel <= debuglevel && packetwin != NULL) {
+    if (errlevel <= context.debug && packetwin != NULL) {
 	va_list ap;
 	va_start(ap, fmt);
 	(void)vsnprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), fmt, ap);
@@ -204,18 +207,15 @@ void gpsd_report(int errlevel, const char *fmt, ...)
     }
 }
 
-/*@ -globstate @*/
 static ssize_t readpkt(void)
 {
-    /*@ -type -shiftnegative -compdef -nullpass @*/
+    /*@ -globstate -type -shiftnegative -compdef -nullpass @*/
     struct timeval timeval;
     fd_set select_set;
     gps_mask_t changed;
 
     FD_ZERO(&select_set);
     FD_SET(session.gpsdata.gps_fd, &select_set);
-    if (controlfd > -1)
-	FD_SET(controlfd, &select_set);
     /*
      * If the timeout on this select isn't longer than the device's
      * cycle time, the code will be prone to flaky timing-dependent
@@ -234,7 +234,13 @@ static ssize_t readpkt(void)
     if (changed == 0)
 	longjmp(terminate, TERM_EMPTY_READ);
 
-    if ((changed & ERROR_IS) != 0)
+    /* conditional prevents mask dumper from eating CPU */
+    if (context.debug >= LOG_DATA)
+	gpsd_report(LOG_DATA,
+		    "packet mask = %s\n",
+		    gps_maskdump(session.gpsdata.set));
+
+    if ((changed & ERROR_SET) != 0)
 	longjmp(terminate, TERM_READ_ERROR);
 
     if (logfile != NULL && session.packet.outbuflen > 0) {
@@ -245,11 +251,9 @@ static ssize_t readpkt(void)
 	/*@ +shiftimplementation +sefparams -charint @*/
     }
     return session.packet.outbuflen;
-    /*@ +type +shiftnegative +compdef +nullpass @*/
+    /*@ +globstate +type +shiftnegative +compdef +nullpass @*/
 }
 
-/*@ +globstate @*/
-
 static void packet_dump(const char *buf, size_t buflen)
 {
     if (packetwin != NULL) {
@@ -273,7 +277,7 @@ static void packet_dump(const char *buf, size_t buflen)
     }
 }
 
-#if defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE)
+#if defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE)
 static void monitor_dump_send(/*@in@*/ const char *buf, size_t len)
 {
     if (packetwin != NULL) {
@@ -297,43 +301,19 @@ static void announce_log(/*@in@*/ const char *str)
        (void)fprintf(logfile, ">>>%s\n", str);
    }
 }
-#endif /* defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE) */
+#endif /* defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE) */
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 bool monitor_control_send( /*@in@*/ unsigned char *buf, size_t len)
 {
-    if ((controlfd == -1) || (session.gpsdata.dev.path[0] == '\0'))
+    if (!serial)
 	return false;
     else {
-	int savefd;
 	ssize_t st;
 
-	if (!serial) {
-	    /*@ -sefparams @*/
-	    assert(write(controlfd, "!", 1) != -1);
-	    assert(write
-		   (controlfd, session.gpsdata.dev.path,
-		    strlen(session.gpsdata.dev.path)) != -1);
-	    assert(write(controlfd, "=", 1) != -1);
-	    /*@ +sefparams @*/
-	    /*
-	     * Ugh...temporarily con the libgpsd layer into using the
-	     * socket descriptor.
-	     */
-	    savefd = session.gpsdata.gps_fd;
-	    session.gpsdata.gps_fd = controlfd;
-	}
-
+	context.readonly = false;
 	st = (*active)->driver->control_send(&session, (char *)buf, len);
-
-	if (!serial) {
-	    /* stop pretending now */
-	    session.gpsdata.gps_fd = controlfd;
-	    /* enough room for "ERROR\r\n\0" */
-	    /*@ -sefparams @*/
-	    assert(read(controlfd, buf, 8) != -1);
-	    /*@ +sefparams @*/
-	}
+	context.readonly = true;
 	monitor_dump_send((const char *)buf, len);
 	return (st != -1);
     }
@@ -341,33 +321,16 @@ bool monitor_control_send( /*@in@*/ unsigned char *buf, size_t len)
 
 static bool monitor_raw_send( /*@in@*/ unsigned char *buf, size_t len)
 {
-    if ((controlfd == -1) || (session.gpsdata.dev.path[0] == '\0'))
+    if (!serial)
 	return false;
     else {
 	ssize_t st;
-
-	if (!serial) {
-	    /*@ -sefparams @*/
-	    assert(write(controlfd, "!", 1) != -1);
-	    assert(write(controlfd, session.gpsdata.dev.path,
-			 strlen(session.gpsdata.dev.path)) != -1);
-	    assert(write(controlfd, "=", 1) != -1);
-	    /*@ +sefparams @*/
-	}
-
-	st = write(controlfd, (char *)buf, len);
-
-	if (!serial) {
-	    /* enough room for "ERROR\r\n\0" */
-	    /*@ -sefparams @*/
-	    assert(read(controlfd, buf, 8) != -1);
-	    /*@ +sefparams @*/
-	}
+	st = write(session.gpsdata.gps_fd, (char *)buf, len);
 	monitor_dump_send((const char *)buf, len);
 	return (st > 0 && (size_t) st == len);
     }
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
 /*****************************************************************************
  *
@@ -389,7 +352,6 @@ void monitor_complain(const char *fmt, ...)
     (void)wgetch(cmdwin);
 }
 
-
 void monitor_log(const char *fmt, ...)
 {
     if (packetwin != NULL) {
@@ -404,9 +366,12 @@ static bool switch_type(const struct gps_type_t *devtype)
 {
     const struct monitor_object_t **trial, **newobject;
     newobject = NULL;
-    for (trial = monitor_objects; *trial; trial++)
-	if ((*trial)->driver == devtype)
+    for (trial = monitor_objects; *trial; trial++) {
+	if (strcmp((*trial)->driver->type_name, devtype->type_name)==0) {
 	    newobject = trial;
+	    break;
+	}
+    }
     if (newobject) {
 	int leftover;
 	if (LINES < (*newobject)->min_y + 1 || COLS < (*newobject)->min_x) {
@@ -414,12 +379,13 @@ static bool switch_type(const struct gps_type_t *devtype)
 			     (*newobject)->min_x, (*newobject)->min_y + 1);
 	} else {
 	    if (active != NULL) {
-		(*active)->wrap();
+		if ((*active)->wrap != NULL)
+		    (*active)->wrap();
 		(void)delwin(devicewin);
 	    }
 	    active = newobject;
 	    devicewin = newwin((*active)->min_y, (*active)->min_x, 1, 0);
-	    if ((devicewin == NULL) || !(*active)->initialize()) {
+	    if ((devicewin == NULL) || ((*active)->initialize != NULL && !(*active)->initialize())) {
 		monitor_complain("Internal initialization failure - screen "
 				 "must be at least 80x24. aborting.");
 		return false;
@@ -445,7 +411,7 @@ static bool switch_type(const struct gps_type_t *devtype)
 	return true;
     }
 
-    monitor_complain("No matching monitor type.");
+    monitor_complain("No monitor matches %s.", devtype->type_name);
     return false;
 }
 
@@ -456,31 +422,30 @@ static void onsig(int sig UNUSED)
     longjmp(assertbuf, 1);
 }
 
+#define WATCHRAW	"?WATCH={\"raw\":2}\r\n"
+#define WATCHRAWDEVICE	"?WATCH={\"raw\":2,\"device\":\"%s\"}\r\n"
+
 int main(int argc, char **argv)
 {
-#if defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE)
+#if defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE)
     unsigned int v;
-#endif /* defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE) */
+#endif /* defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE) */
     int option, status, last_type = BAD_PACKET;
     ssize_t len;
     struct fixsource_t source;
-    char *p, *controlsock = "/var/run/gpsd.sock";
     fd_set select_set;
     unsigned char buf[BUFLEN];
-    char line[80], *explanation;
+    char line[80], *explanation, *p;
     int bailout = 0, matches = 0;
 
     /*@ -observertrans @*/
     (void)putenv("TZ=UTC");	// for ctime()
     /*@ +observertrans @*/
     /*@ -branchstate @*/
-    while ((option = getopt(argc, argv, "D:F:LVhl:t:?")) != -1) {
+    while ((option = getopt(argc, argv, "D:LVhl:t:?")) != -1) {
 	switch (option) {
 	case 'D':
-	    debuglevel = atoi(optarg);
-	    break;
-	case 'F':
-	    controlsock = optarg;
+	    context.debug = atoi(optarg);
 	    break;
 	case 'L':		/* list known device types */
 	    (void)
@@ -490,7 +455,7 @@ int main(int argc, char **argv)
 	    for (active = monitor_objects; *active; active++) {
 		(void)fputs("i l q ^S ^Q", stdout);
 		(void)fputc(' ', stdout);
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 		if ((*active)->driver->mode_switcher != NULL)
 		    (void)fputc('n', stdout);
 		else
@@ -506,13 +471,13 @@ int main(int argc, char **argv)
 		else
 		    (void)fputc(' ', stdout);
 		(void)fputc(' ', stdout);
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
 		if ((*active)->driver->control_send != NULL)
 		    (void)fputc('x', stdout);
 		else
 		    (void)fputc(' ', stdout);
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 		(void)fputc(' ', stdout);
 		if ((*active)->command != NULL)
 		    (void)fputc('+', stdout);
@@ -557,7 +522,7 @@ int main(int argc, char **argv)
 	default:
 	    (void)
 		fputs
-		("usage:  gpsmon [-?hVl] [-D debuglevel] [-F controlsock] [-t type] [server[:port:[device]]]\n",
+		("usage:  gpsmon [-?hVl] [-D debuglevel] [-t type] [server[:port:[device]]]\n",
 		 stderr);
 	    exit(1);
 	}
@@ -589,11 +554,8 @@ int main(int argc, char **argv)
 			  netlib_errstr(session.gpsdata.gps_fd));
 	    exit(1);
 	}
-	controlfd = open(controlsock, O_RDWR);
 	if (source.device != NULL) {
-	    (void)gps_send(&session.gpsdata,
-			   "?WATCH={\"raw\":2,\"device\":\"%s\"}\r\n",
-			   source.device);
+	    (void)gps_send(&session.gpsdata, WATCHRAWDEVICE, source.device);
 	    /*
 	     *  The gpsdata.dev is filled only in JSON mode,
 	     *  but we in super-raw mode.
@@ -601,7 +563,7 @@ int main(int argc, char **argv)
 	    (void)strlcpy(session.gpsdata.dev.path, source.device,
 			  sizeof(session.gpsdata.dev.path));
 	} else {
-	    (void)gps_send(&session.gpsdata, "?WATCH={\"raw\":2}\r\n");
+	    (void)gps_send(&session.gpsdata, WATCHRAW);
 	    session.gpsdata.dev.path[0] = '\0';
 	}
 	serial = false;
@@ -615,7 +577,6 @@ int main(int argc, char **argv)
 	    exit(2);
 	}
 
-	controlfd = session.gpsdata.gps_fd;
 	serial = true;
     }
     /*@ +boolops */
@@ -700,7 +661,9 @@ int main(int argc, char **argv)
 		(void)wprintw(cmdwin, type_name);
 		(void)wprintw(cmdwin, "> ");
 		(void)wclrtoeol(cmdwin);
-		if (active != NULL && len > 0 && session.packet.outbuflen > 0)
+		if (active != NULL 
+			&& len > 0 && session.packet.outbuflen > 0
+			&& (*active)->update != NULL)
 		    (*active)->update();
 		(void)wprintw(packetwin, "(%d) ", session.packet.outbuflen);
 		packet_dump((char *)session.packet.outbuffer,
@@ -740,7 +703,6 @@ int main(int argc, char **argv)
 		    continue;
 		/*@ +usedef +compdef @*/
 
-		arg = line;
 		if (isspace(line[1])) {
 		    for (arg = line + 2; *arg != '\0' && isspace(*arg); arg++)
 			arg++;
@@ -748,7 +710,7 @@ int main(int argc, char **argv)
 		} else
 		    arg = line + 1;
 
-		if (active != NULL && (*active)->command != NULL) {
+		if (serial && active != NULL && (*active)->command != NULL) {
 		    status = (*active)->command(line);
 		    if (status == COMMAND_TERMINATE)
 			goto quit;
@@ -757,22 +719,19 @@ int main(int argc, char **argv)
 		    assert(status == COMMAND_UNKNOWN);
 		}
 		switch (line[0]) {
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 		case 'c':	/* change cycle time */
 		    if (active == NULL)
 			monitor_complain("No device defined yet");
-		    else if (serial) {
+		    else if (!serial)
+			monitor_complain("Only available in low-level mode.");
+		    else {
 			double rate = strtod(arg, NULL);
 			const struct monitor_object_t **switcher = active;
 
 			if (fallback != NULL && (*fallback)->driver->rate_switcher != NULL)
 			    switcher = fallback;
-			/* Ugh...should have a controlfd slot
-			 * in the session structure, really
-			 */
 			if ((*switcher)->driver->rate_switcher) {
-			    int dfd = session.gpsdata.gps_fd;
-			    session.gpsdata.gps_fd = controlfd;
 			    /* *INDENT-OFF* */
 			    context.readonly = false;
 			    if ((*switcher)->driver->rate_switcher(&session, rate)) {
@@ -781,27 +740,17 @@ int main(int argc, char **argv)
 				monitor_complain("Rate not supported.");
 			    context.readonly = true;
 			    /* *INDENT-ON* */
-			    session.gpsdata.gps_fd = dfd;
 			} else
 			    monitor_complain
 				("Device type has no rate switcher");
-		    } else {
-			line[0] = 'c';
-			/*@ -sefparams @*/
-			assert(write
-			       (session.gpsdata.gps_fd, line,
-				strlen(line)) != -1);
-			/* discard response */
-			assert(read(session.gpsdata.gps_fd, buf, sizeof(buf))
-			       != -1);
-			/*@ +sefparams @*/
 		    }
+#endif /* RECONFIGURE_ENABLE */
 		    break;
-#endif /* ALLOW_RECONFIGURE */
-
 		case 'i':	/* start probing for subtype */
 		    if (active == NULL)
 			monitor_complain("No GPS type detected.");
+		    else if (!serial)
+			monitor_complain("Only available in low-level mode.");
 		    else {
 			if (strcspn(line, "01") == strlen(line))
 			    context.readonly = !context.readonly;
@@ -828,7 +777,7 @@ int main(int argc, char **argv)
 					  ">>> Logging to %s on", logfile);
 		    break;
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 		case 'n':	/* change mode */
 		    /* if argument not specified, toggle */
 		    if (strcspn(line, "01") == strlen(line)) {
@@ -840,17 +789,14 @@ int main(int argc, char **argv)
 			v = (unsigned)atoi(line + 1);
 		    if (active == NULL)
 			monitor_complain("No device defined yet");
-		    else if (serial) {
+		    else if (!serial)
+			monitor_complain("Only available in low-level mode.");
+		    else {
 			const struct monitor_object_t **switcher = active;
 
 			if (fallback != NULL && (*fallback)->driver->mode_switcher != NULL)
 			    switcher = fallback;
-			/* Ugh...should have a controlfd slot
-			 * in the session structure, really
-			 */
 			if ((*switcher)->driver->mode_switcher) {
-			    int dfd = session.gpsdata.gps_fd;
-			    session.gpsdata.gps_fd = controlfd;
 			    context.readonly = false;
 			    (*switcher)->driver->mode_switcher(&session,
 							     (int)v);
@@ -858,34 +804,23 @@ int main(int argc, char **argv)
 			    announce_log("Mode switcher called");
 			    (void)tcdrain(session.gpsdata.gps_fd);
 			    (void)usleep(50000);
-			    session.gpsdata.gps_fd = dfd;
 			} else
 			    monitor_complain
 				("Device type has no mode switcher");
-		    } else {
-			line[0] = 'n';
-			line[1] = ' ';
-			line[2] = '0' + v;
-			/*@ -sefparams @*/
-			assert(write
-			       (session.gpsdata.gps_fd, line,
-				strlen(line)) != -1);
-			/* discard response */
-			assert(read(session.gpsdata.gps_fd, buf, sizeof(buf))
-			       != -1);
-			/*@ +sefparams @*/
 		    }
 		    break;
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 		case 'q':	/* quit */
 		    goto quit;
 
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 		case 's':	/* change speed */
 		    if (active == NULL)
 			monitor_complain("No device defined yet");
-		    else if (serial) {
+		    else if (!serial)
+			monitor_complain("Only available in low-level mode.");
+		    else {
 			speed_t speed;
 			char parity = session.gpsdata.dev.parity;
 			unsigned int stopbits =
@@ -919,13 +854,8 @@ int main(int argc, char **argv)
 			}
 			/*@ -charint @*/
 			speed = (unsigned)atoi(arg);
-			/* Ugh...should have a controlfd slot
-			 * in the session structure, really
-			 */
 			/* *INDENT-OFF* */
 			if ((*switcher)->driver->speed_switcher) {
-			    int dfd = session.gpsdata.gps_fd;
-			    session.gpsdata.gps_fd = controlfd;
 			    context.readonly = false;
 			    if ((*switcher)->
 				driver->speed_switcher(&session, speed,
@@ -933,7 +863,7 @@ int main(int argc, char **argv)
 						       stopbits)) {
 				announce_log("Speed switcher called.");
 				/*
-				 * See the comment attached to the 'B'
+				 * See the comment attached to the 'DEVICE'
 				 * command in gpsd.  Allow the control
 				 * string time to register at the GPS
 				 * before we do the baud rate switch,
@@ -948,27 +878,18 @@ int main(int argc, char **argv)
 				monitor_complain
 				    ("Speed/mode combination not supported.");
 			    context.readonly = true;
-			    session.gpsdata.gps_fd = dfd;
 			} else
 			    monitor_complain
 				("Device type has no speed switcher");
 			/* *INDENT-ON* */
-		    } else {
-			line[0] = 'b';
-			/*@ -sefparams @*/
-			assert(write
-			       (session.gpsdata.gps_fd, line,
-				strlen(line)) != -1);
-			/* discard response */
-			assert(read(session.gpsdata.gps_fd, buf, sizeof(buf))
-			       != -1);
-			/*@ +sefparams @*/
 		    }
 		    break;
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 
 		case 't':	/* force device type */
-		    if (strlen(arg) > 0) {
+		    if (!serial)
+			monitor_complain("Only available in low-level mode.");
+		    else if (strlen(arg) > 0) {
 			int matchcount = 0;
 			const struct gps_type_t **dp, *forcetype = NULL;
 			for (dp = gpsd_drivers; *dp; dp++) {
@@ -995,10 +916,12 @@ int main(int argc, char **argv)
 		    }
 		    break;
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 		case 'x':	/* send control packet */
 		    if (active == NULL)
 			monitor_complain("No device defined yet");
+		    else if (!serial)
+			monitor_complain("Only available in low-level mode.");
 		    else {
 			/*@ -compdef @*/
 			int st = gpsd_hexpack(arg, (char *)buf, strlen(arg));
@@ -1010,22 +933,26 @@ int main(int argc, char **argv)
 				("Device type has no control-send method.");
 			else if (!monitor_control_send(buf, (size_t) st))
 			    monitor_complain("Control send failed.");
+			/*@ +compdef @*/
 		    }
-		    /*@ +compdef @*/
 		    break;
 
 		case 'X':	/* send raw packet */
-		    /*@ -compdef @*/
-		    len =
-			(ssize_t) gpsd_hexpack(arg, (char *)buf, strlen(arg));
-		    if (len < 0)
-			monitor_complain("Invalid hex string (error %d)",
-					 len);
-		    else if (!monitor_raw_send(buf, (size_t) len))
-			monitor_complain("Raw send failed.");
-		    /*@ +compdef @*/
+		    if (!serial)
+			monitor_complain("Only available in low-level mode.");
+		    else {
+			/*@ -compdef @*/
+			len =
+			    (ssize_t) gpsd_hexpack(arg, (char *)buf, strlen(arg));
+			if (len < 0)
+			    monitor_complain("Invalid hex string (error %d)",
+					     len);
+			else if (!monitor_raw_send(buf, (size_t) len))
+			    monitor_complain("Raw send failed.");
+			/*@ +compdef @*/
+		    }
 		    break;
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
 		default:
 		    monitor_complain("Unknown command");
