@@ -38,6 +38,12 @@ void gps_clear_fix( /*@out@*/ struct gps_fix_t *fixp)
     fixp->epc = NAN;
 }
 
+void gps_clear_dop( /*@out@*/ struct dop_t *dop)
+{
+    dop->xdop = dop->ydop = dop->vdop = dop->tdop = dop->hdop = dop->pdop =
+        dop->gdop = NAN;
+}
+
 void gps_merge_fix( /*@ out @*/ struct gps_fix_t *to,
 		   gps_mask_t transfer,
 		   /*@ in @*/ struct gps_fix_t *from)
@@ -45,31 +51,31 @@ void gps_merge_fix( /*@ out @*/ struct gps_fix_t *to,
 {
     if ((NULL == to) || (NULL == from))
 	return;
-    if ((transfer & TIME_IS) != 0)
+    if ((transfer & TIME_SET) != 0)
 	to->time = from->time;
-    if ((transfer & LATLON_IS) != 0) {
+    if ((transfer & LATLON_SET) != 0) {
 	to->latitude = from->latitude;
 	to->longitude = from->longitude;
     }
-    if ((transfer & MODE_IS) != 0)
+    if ((transfer & MODE_SET) != 0)
 	to->mode = from->mode;
-    if ((transfer & ALTITUDE_IS) != 0)
+    if ((transfer & ALTITUDE_SET) != 0)
 	to->altitude = from->altitude;
-    if ((transfer & TRACK_IS) != 0)
+    if ((transfer & TRACK_SET) != 0)
 	to->track = from->track;
-    if ((transfer & SPEED_IS) != 0)
+    if ((transfer & SPEED_SET) != 0)
 	to->speed = from->speed;
-    if ((transfer & CLIMB_IS) != 0)
+    if ((transfer & CLIMB_SET) != 0)
 	to->climb = from->climb;
-    if ((transfer & TIMERR_IS) != 0)
+    if ((transfer & TIMERR_SET) != 0)
 	to->ept = from->ept;
-    if ((transfer & HERR_IS) != 0) {
+    if ((transfer & HERR_SET) != 0) {
 	to->epx = from->epx;
 	to->epy = from->epy;
     }
-    if ((transfer & VERR_IS) != 0)
+    if ((transfer & VERR_SET) != 0)
 	to->epv = from->epv;
-    if ((transfer & SPEEDERR_IS) != 0)
+    if ((transfer & SPEEDERR_SET) != 0)
 	to->eps = from->eps;
 }
 
@@ -138,7 +144,7 @@ timestamp_t iso8601_to_unix( /*@in@*/ char *isotime)
 /*@observer@*/char *unix_to_iso8601(timestamp_t fixtime, /*@ out @*/
 				     char isotime[], size_t len)
 /* Unix UTC time to ISO8601, no timezone adjustment */
-/* example: 2007-12-11T23:38:51.03Z */
+/* example: 2007-12-11T23:38:51.033Z */
 {
     struct tm when;
     double integral, fractional;
@@ -151,7 +157,12 @@ timestamp_t iso8601_to_unix( /*@in@*/ char *isotime)
     (void)gmtime_r(&intfixtime, &when);
 
     (void)strftime(timestr, sizeof(timestr), "%Y-%m-%dT%H:%M:%S", &when);
-    (void)snprintf(fractstr, sizeof(fractstr), "%.2f", fractional);
+    /*
+     * Do not mess casually with the number of decimal digits in the
+     * format!  Most GPSes report over serial links at 0.01s or 0.001s
+     * precision.
+     */
+    (void)snprintf(fractstr, sizeof(fractstr), "%.3f", fractional);
     /* add fractional part, ignore leading 0; "0.2" -> ".2" */
     /*@i2@*/(void)snprintf(isotime, len, "%s%sZ",timestr, strchr(fractstr,'.'));
     return isotime;
