@@ -3,53 +3,47 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 #include <string.h>
+#include <ctype.h>
+#include <assert.h>
 
 #include "gpsd.h"
 
-int gpsd_hexdump_level = -1;
-/*
- * A wrapper around gpsd_hexdump to prevent wasting cpu time by hexdumping
- * buffers and copying strings that will never be printed. only messages at
- * level "N" and lower will be printed. By way of example, without any -D
- * options, gpsd probably won't ever call the real gpsd_hexdump. At -D2,
- * LOG_PROG (and higher) won't get to call the real gpsd_hexdump. For high
- * speed, chatty protocols, this can save a lot of CPU.
- */
-char *gpsd_hexdump_wrapper(const void *binbuf, size_t binbuflen,
-			   int msg_debug_level)
+char /*@ observer @*/ *gpsd_hexdump(char *binbuf, size_t binbuflen)
 {
+    char *cp;
+    bool printable = true;
+
+    assert(binbuf != NULL);
+    for (cp = binbuf; cp < binbuf + binbuflen; cp++)
+	if (!isprint(*cp) && !isspace(*cp))
+	    printable = false;
+    if (printable)
+	return binbuf;
+    else {
+	static char hexbuf[MAX_PACKET_LENGTH * 2 + 1];
 #ifndef SQUELCH_ENABLE
-    if (msg_debug_level <= gpsd_hexdump_level)
-	return gpsd_hexdump(binbuf, binbuflen);
-#endif /* SQUELCH_ENABLE */
-    return "";
-}
+	size_t i, j = 0;
+	size_t len =
+	    (size_t) ((binbuflen >
+		       MAX_PACKET_LENGTH) ? MAX_PACKET_LENGTH : binbuflen);
+	const char *ibuf = (const char *)binbuf;
+	const char *hexchar = "0123456789abcdef";
 
-char /*@ observer @*/ *gpsd_hexdump(const void *binbuf, size_t binbuflen)
-{
-    static char hexbuf[MAX_PACKET_LENGTH * 2 + 1];
-#ifndef SQUELCH_ENABLE
-    size_t i, j = 0;
-    size_t len =
-	(size_t) ((binbuflen >
-		   MAX_PACKET_LENGTH) ? MAX_PACKET_LENGTH : binbuflen);
-    const char *ibuf = (const char *)binbuf;
-    const char *hexchar = "0123456789abcdef";
+	if (NULL == binbuf || 0 == binbuflen)
+	    return "";
 
-    if (NULL == binbuf || 0 == binbuflen)
-	return "";
-
-    /*@ -shiftimplementation @*/
-    for (i = 0; i < len; i++) {
-	hexbuf[j++] = hexchar[(ibuf[i] & 0xf0) >> 4];
-	hexbuf[j++] = hexchar[ibuf[i] & 0x0f];
-    }
-    /*@ +shiftimplementation @*/
-    hexbuf[j] = '\0';
+	/*@ -shiftimplementation @*/
+	for (i = 0; i < len; i++) {
+	    hexbuf[j++] = hexchar[(ibuf[i] & 0xf0) >> 4];
+	    hexbuf[j++] = hexchar[ibuf[i] & 0x0f];
+	}
+	/*@ +shiftimplementation @*/
+	hexbuf[j] = '\0';
 #else /* SQUELCH defined */
-    hexbuf[0] = '\0';
+	hexbuf[0] = '\0';
 #endif /* SQUELCH_ENABLE */
-    return hexbuf;
+	return hexbuf;
+    }
 }
 
 int gpsd_hexpack( /*@in@*/ const char *src, /*@out@ */ char *dst, size_t len)
