@@ -67,8 +67,11 @@ PERMISSIONS
 #include <ctype.h>
 
 #include "gpsd_config.h"	/* for strlcpy() prototype */
+#ifdef SOCKET_EXPORT_ENABLE
 #include "json.h"
 
+#include "gps.h"		/* only for timestamp_t prototype */
+
 #ifdef CLIENTDEBUG_ENABLE
 static int debuglevel = 0;
 static FILE *debugfp;
@@ -118,6 +121,7 @@ static /*@null@*/ char *json_target_address(const struct json_attr_t *cursor,
 	case t_uinteger:
 	    targetaddr = (char *)&cursor->addr.uinteger[offset];
 	    break;
+	case t_time:
 	case t_real:
 	    targetaddr = (char *)&cursor->addr.real[offset];
 	    break;
@@ -195,6 +199,7 @@ static int json_internal_read_object(const char *cp,
 		case t_uinteger:
 		    *((unsigned int *)lptr) = cursor->dflt.uinteger;
 		    break;
+		case t_time:
 		case t_real:
 		    *((double *)lptr) = cursor->dflt.real;
 		    break;
@@ -276,6 +281,8 @@ static int json_internal_read_object(const char *cp,
 		    maxlen = (int)cursor->len - 1;
 		else if (cursor->type == t_check)
 		    maxlen = (int)strlen(cursor->dflt.check);
+		else if (cursor->type == t_time)
+		    maxlen = JSON_VAL_MAX;
 		else if (cursor->map != NULL)
 		    maxlen = (int)sizeof(valbuf) - 1;
 		pval = valbuf;
@@ -381,7 +388,7 @@ static int json_internal_read_object(const char *cp,
 	     */
 	    for (;;) {
 		int seeking = cursor->type;
-		if (value_quoted && cursor->type == t_string)
+		if (value_quoted && (cursor->type == t_string || cursor->type == t_time))
 		    break;
 		if ((strcmp(valbuf, "true")==0 || strcmp(valbuf, "false")==0)
 			&& seeking == t_boolean)
@@ -401,14 +408,15 @@ static int json_internal_read_object(const char *cp,
 	    }
 	    if (value_quoted
 		&& (cursor->type != t_string && cursor->type != t_character
-		    && cursor->type != t_check && cursor->map == 0)) {
+		    && cursor->type != t_check && cursor->type != t_time 
+		    && cursor->map == 0)) {
 		json_debug_trace((1,
 				  "Saw quoted value when expecting non-string.\n"));
 		return JSON_ERR_QNONSTRING;
 	    }
 	    if (!value_quoted
 		&& (cursor->type == t_string || cursor->type == t_check
-		    || cursor->map != 0)) {
+		    || cursor->type == t_time || cursor->map != 0)) {
 		json_debug_trace((1,
 				  "Didn't see quoted value when expecting string.\n"));
 		return JSON_ERR_NONQSTRING;
@@ -433,6 +441,9 @@ static int json_internal_read_object(const char *cp,
 		case t_uinteger:
 		    *((unsigned int *)lptr) = (unsigned)atoi(valbuf);
 		    break;
+		case t_time:
+		    *((double *)lptr) = iso8601_to_unix(valbuf);
+		    break;
 		case t_real:
 		    *((double *)lptr) = atof(valbuf);
 		    break;
@@ -441,7 +452,7 @@ static int json_internal_read_object(const char *cp,
 			&& parent->element_type != t_structobject
 			&& offset > 0)
 			return JSON_ERR_NOPARSTR;
-		    (void)strncpy(lptr, valbuf, cursor->len);
+		    (void)strlcpy(lptr, valbuf, cursor->len);
 		    break;
 		case t_boolean:
 		    *((bool *) lptr) = (strcmp(valbuf, "true") == 0);
@@ -497,7 +508,7 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
 		    const char **end)
 {
     /*@-nullstate -onlytrans@*/
-    int substatus, offset;
+    int substatus, offset, arrcount;
     char *tp;
 
     if (end != NULL)
@@ -514,8 +525,7 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
 	cp++;
 
     tp = arr->arr.strings.store;
-    if (arr->count != NULL)
-	*(arr->count) = 0;
+    arrcount = 0;
     for (offset = 0; offset < arr->maxlen; offset++) {
 	json_debug_trace((1, "Looking at %s\n", cp));
 	switch (arr->element_type) {
@@ -554,6 +564,7 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
 	    break;
 	case t_integer:
 	case t_uinteger:
+	case t_time:
 	case t_real:
 	case t_boolean:
 	case t_character:
@@ -562,8 +573,7 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
 	    json_debug_trace((1, "Invalid array subtype.\n"));
 	    return JSON_ERR_SUBTYPE;
 	}
-	if (arr->count != NULL)
-	    (*arr->count)++;
+	arrcount++;
 	if (isspace(*cp))
 	    cp++;
 	if (*cp == ']') {
@@ -579,11 +589,13 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
     json_debug_trace((1, "Too many elements in array.\n"));
     return JSON_ERR_SUBTOOLONG;
   breakout:
+    if (arr->count != NULL)
+	*(arr->count) = arrcount;
     if (end != NULL)
 	*end = cp;
     /*@ -nullderef @*/
     json_debug_trace((1, "leaving json_read_array() with %d elements\n",
-		      *arr->count));
+		      arrcount));
     /*@ +nullderef @*/
     return 0;
     /*@+nullstate +onlytrans@*/
@@ -595,6 +607,7 @@ int json_read_object(const char *cp, const struct json_attr_t *attrs,
     char *savedlocale = setlocale(LC_ALL, NULL);
     int st;
 
+    /* temporary locale setting is required wherever ',' is the decimal point */
     (void)setlocale(LC_ALL, "C");
     json_debug_trace((1, "json_read_object() sees '%s'\n", cp));
     st = json_internal_read_object(cp, attrs, NULL, 0, end);
@@ -634,3 +647,8 @@ const /*@observer@*/ char *json_error_string(int err)
     else
 	return errors[err];
 }
+
+#endif /* SOCKET_EXPORT_ENABLE */
+
+/* end */
+
