@@ -23,7 +23,7 @@
 # This file is Copyright (c) 2010 by the GPSD project
 # BSD terms apply: see the file COPYING in the distribution root for details.
 #
-import os, urllib, re, random, time, calendar, math
+import os, urllib, re, random, time, calendar, math, sys
 
 __locations = [
     (
@@ -113,12 +113,13 @@ def save_leapseconds(outfile):
     "Fetch the USNO leap-second history data and make a leap-second list."
     skip = True
     leapsecs = []
-    # This code assumes that after 1980, leap-second increments are
-    # always integrally one second and every increment is listed here
-    leapsecs = []
     try:
+        fetchobj = urllib.urlopen("ftp://maia.usno.navy.mil/ser7/tai-utc.dat")
+        # This code assumes that after 1980, leap-second increments are
+        # always integrally one second and every increment is listed here
+        leapsecs = []
         fp = open(outfile, "w")
-        for line in urllib.urlopen("ftp://maia.usno.navy.mil/ser7/tai-utc.dat"):
+        for line in fetchobj:
             if line.startswith(" 1980"):
                 skip = False
             if skip:
@@ -133,7 +134,7 @@ def save_leapseconds(outfile):
 def fetch_leapsecs(filename):
     "Get a list of leap seconds from the local cache of the USNO history"
     leapsecs = []
-    for line in open(filename):
+    for line in open(str(filename)):
         leapsecs.append(float(line.strip()))
     return leapsecs
 
@@ -146,8 +147,7 @@ def make_leapsecond_include(infile):
         else:
             return str(i)
     year = time.strftime("%Y", time.localtime(time.time()))
-    sys.stdout.write("#define CENTURY_BASE\t%s00\n" % year[:2])
-    sys.stdout.write("#define LEAPSECOND_NOW\t%d\n" % (len(leapsecs)-2)) 
+    return ("#define CENTURY_BASE\t%s00\n" % year[:2]) + ("#define LEAPSECOND_NOW\t%d\n" % (len(leapsecs)-2))
 
 def leastsquares(tuples):
     "Generate coefficients for a least-squares fit to the specified data."
@@ -252,7 +252,7 @@ if __name__ == '__main__':
             graph_history(val)
             raise SystemExit, 0
         elif (switch == '-h'):  # make leapsecond include
-            make_leapsecond_include(val)
+            sys.stdout.write(make_leapsecond_include(val))
             raise SystemExit, 0
         elif (switch == '-i'):  # Compute Unix time from RFC822 date
             print rfc822_to_unix(val)
