@@ -1,4 +1,9 @@
-/* libgpsd_core.c -- direct access to GPSes on serial or USB devices.
+/* libgpsd_core.c -- manage access to sensors
+ *
+ * Access to the driver layer goes through the entry points in this file.
+ * The idea is to present a session as an abstraction from which you get 
+ * fixes (and possibly other data updates) by calling gpsd_poll(). The
+ * rest is setup and teardown.
  *
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
@@ -21,89 +26,12 @@
 
 #include "gpsd.h"
 
-#if defined(PPS_ENABLE)
-/*
- * Warning: This is a potential portability problem. 
- * It's needed so that TIOCMIWAIT will be defined and the serial-PPS 
- * code will work, but it's not a SuS/POSIX standard header.  We're
- * going to include it unconditionally here because we expect both
- * Linux and BSD to have it and we want compilation to break with
- * an audible snapping 
- */
-#include <sys/ioctl.h>
-
-#ifndef S_SPLINT_S
-#include <pthread.h>		/* pacifies OpenBSD's compiler */
-#endif
-#if defined(HAVE_SYS_TIMEPPS_H)
-    /* use RFC 2783 PPS API */
-    /* this needs linux >= 2.6.34 and
-     * CONFIG_PPS=y
-     * CONFIG_PPS_DEBUG=y  [optional to kernel log pulses]
-     * CONFIG_PPS_CLIENT_LDISC=y
-     */
-    /* get timepps.h from the pps-tools package, or from here:
-     * http://www.mail-archive.com/debian-glibc@lists.debian.org/msg43125.html
-     * RFC2783 says timepps.h is in sys
-     */
-    #include <sys/timepps.h>
-    #include <glob.h>
-#endif
-/* and for chrony */
-#include <sys/un.h>
-
-/* normalize a timespec */
-#define TS_NORM(ts)  \
-    do { \
-	if ( 1000000000 <= (ts)->tv_nsec ) { \
-	    (ts)->tv_nsec -= 1000000000; \
-	    (ts)->tv_sec++; \
-	} else if ( 0 > (ts)->tv_nsec ) { \
-	    (ts)->tv_nsec += 1000000000; \
-	    (ts)->tv_sec--; \
-	} \
-    } while (0)
-
-/* normalize a timeval */
-#define TV_NORM(tv)  \
-    do { \
-	if ( 1000000 <= (tv)->tv_usec ) { \
-	    (tv)->tv_usec -= 1000000; \
-	    (tv)->tv_sec++; \
-	} else if ( 0 > (tv)->tv_usec ) { \
-	    (tv)->tv_usec += 1000000; \
-	    (tv)->tv_sec--; \
-	} \
-    } while (0)
-
-/* convert timeval to timespec, with rounding */
-#define TSTOTV(tv, ts) \
-    do { \
-	(tv)->tv_sec = (ts)->tv_sec; \
-	(tv)->tv_usec = ((ts)->tv_nsec + 500)/1000; \
-        TV_NORM( tv ); \
-    } while (0)
-
-/* convert timespec to timeval */
-#define TVTOTS(ts, tv) \
-    do { \
-	(ts)->tv_sec = (tv)->tv_sec; \
-	(ts)->tv_nsec = (tv)->tv_usec*1000; \
-        TS_NORM( ts ); \
-    } while (0)
-
-#endif
-
-#if defined(ONCORE_ENABLE) && defined(BINARY_ENABLE)
-extern const struct gps_type_t oncore_binary;
-#endif
-
 static void gpsd_run_device_hook(char *device_name, char *hook)
 {
     struct stat statbuf;
     if (stat(DEVICEHOOKPATH, &statbuf) == -1)
-	gpsd_report(LOG_PROG, "no %s present, skipped running hook\n",
-	    DEVICEHOOKPATH); 
+	gpsd_report(LOG_PROG, "no %s present, skipped running %s hook\n",
+		    DEVICEHOOKPATH, hook); 
     else {
 	char buf[PATH_MAX];
 	int status;
@@ -124,10 +52,10 @@ int gpsd_switch_driver(struct gps_device_t *session, char *type_name)
     const struct gps_type_t **dp;
     bool identified = (session->device_type != NULL);
 
-    gpsd_report(LOG_PROG, "switch_driver(%s) called...\n", type_name);
     if (identified && strcmp(session->device_type->type_name, type_name) == 0)
 	return 0;
 
+    gpsd_report(LOG_PROG, "switch_driver(%s) called...\n", type_name);
     /*@ -compmempass @*/
     for (dp = gpsd_drivers; *dp; dp++)
 	if (strcmp((*dp)->type_name, type_name) == 0) {
@@ -135,9 +63,9 @@ int gpsd_switch_driver(struct gps_device_t *session, char *type_name)
 			(*dp)->type_name);
 	    gpsd_assert_sync(session);
 	    /*@i@*/ session->device_type = *dp;
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
 	    session->gpsdata.dev.mincycle = session->device_type->min_cycle;
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
 	    /* reconfiguration might be required */
 	    if (identified && session->device_type->event_hook != NULL)
 		session->device_type->event_hook(session,
@@ -157,26 +85,30 @@ void gps_context_init(struct gps_context_t *context)
     /* *INDENT-OFF* */
     /*@ -initallelements -nullassign -nullderef @*/
     struct gps_context_t nullcontext = {
-	.valid	    = 0,
-	.readonly	    = false,
-	.fixcnt	    = 0,
-	.rtcmbytes	    = 0,
-	.rtcmbuf	    = {'\0'},
-	.rtcmtime	    = 0,
+	.valid	        = 0,
+	.debug	        = 0,
+	.readonly	= false,
+	.fixcnt	        = 0,
+	.rtcmbytes	= 0,
+	.rtcmbuf	= {'\0'},
+	.rtcmtime	= 0,
 	.start_time     = 0,
 	.leap_seconds   = 0,
-	.gps_week	    = 0,
+	.gps_week	= 0,
 	.gps_tow        = 0,
-	.century	    = 0,
+	.century	= 0,
 	.rollovers      = 0,
 #ifdef NTPSHM_ENABLE
 	.enable_ntpshm  = false,
-	.shmTime	    = {0},
+	.shmTime	= {0},
 	.shmTimeInuse   = {0},
 # ifdef PPS_ENABLE
 	.shmTimePPS	    = false,
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
+#ifdef SHM_EXPORT_ENABLE
+	.shmexport      = NULL,
+#endif /* SHM_EXPORT_ENABLE */
     };
     /*@ +initallelements +nullassign +nullderef @*/
     /* *INDENT-ON* */
@@ -207,11 +139,7 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context,
     gps_clear_fix(&session->newdata);
     gps_clear_fix(&session->oldfix);
     session->gpsdata.set = 0;
-    session->gpsdata.dop.hdop = NAN;
-    session->gpsdata.dop.vdop = NAN;
-    session->gpsdata.dop.pdop = NAN;
-    session->gpsdata.dop.tdop = NAN;
-    session->gpsdata.dop.gdop = NAN;
+    gps_clear_dop(&session->gpsdata.dop);
     session->gpsdata.epe = NAN;
     session->mag_var = NAN;
     session->gpsdata.dev.cycle = session->gpsdata.dev.mincycle = 1;
@@ -228,15 +156,7 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context,
 void gpsd_deactivate(struct gps_device_t *session)
 /* temporarily release the GPS device */
 {
-#ifdef NTPSHM_ENABLE
-    (void)ntpshm_free(session->context, session->shmindex);
-    session->shmindex = -1;
-# ifdef PPS_ENABLE
-    (void)ntpshm_free(session->context, session->shmTimeP);
-    session->shmTimeP = -1;
-# endif	/* PPS_ENABLE */
-#endif /* NTPSHM_ENABLE */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     if (!session->context->readonly
 	&& session->device_type != NULL
 	&& session->device_type->event_hook != NULL) {
@@ -247,464 +167,13 @@ void gpsd_deactivate(struct gps_device_t *session)
 	    && session->device_type->mode_switcher != NULL)
 	    session->device_type->mode_switcher(session, 0);
     }
-#endif /* ALLOW_RECONFIGURE */
+#endif /* RECONFIGURE_ENABLE */
     gpsd_report(LOG_INF, "closing GPS=%s (%d)\n",
 		session->gpsdata.dev.path, session->gpsdata.gps_fd);
     (void)gpsd_close(session);
     gpsd_run_device_hook(session->gpsdata.dev.path, "DEACTIVATE");
 }
 
-#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-#if defined(HAVE_SYS_TIMEPPS_H)
-/* return handle for kernel pps, or -1 */
-static int init_kernel_pps(struct gps_device_t *session) {
-    int kernelpps_handle = -1;
-    int ldisc = 18;   /* the PPS line discipline */
-    pps_params_t pp;
-    glob_t globbuf;
-    int i;
-    char pps_num = 0;     /* /dev/pps[pps_num] is our device */
-    char path[GPS_PATH_MAX] = "";
-
-    if ( !isatty(session->gpsdata.gps_fd) ) {
-	gpsd_report(LOG_INF, "KPPS gps_fd not a tty\n");
-    	return -1;
-    }
-    /* Attach the line PPS discpline, so no need to ldattach */
-    /* This activates the magic /dev/pps0 device */
-    if ( 0 > ioctl(session->gpsdata.gps_fd, TIOCSETD, &ldisc)) {
-	gpsd_report(LOG_INF, "KPPS cannot set PPS line discipline: %d\n"
-	    , errno);
-    	return -1;
-    }
-
-
-    /* uh, oh, magic file names!, this is not how RFC2783 was designed */
-    /* need to look in /sys/devices/virtual/pps/pps?/path
-     * (/sys/class/pps/pps?/path is just a link to that)
-     * to find the /dev/pps? that matches our serial port.
-     * this code fails if there are more then 10 pps devices.
-     *     
-     * yes, this could be done with libsysfs, but trying to keep the
-     * number of required libs small */
-    memset( (void *)&globbuf, 0, sizeof(globbuf));
-    glob("/sys/devices/virtual/pps/pps?/path", 0, NULL, &globbuf);
-
-    memset( (void *)&path, 0, sizeof(path));
-    for ( i = 0; i < globbuf.gl_pathc; i++ ) {
-        int fd = open(globbuf.gl_pathv[i], O_RDONLY);
-	if ( 0 <= fd ) {
-	    ssize_t r = read( fd, path, sizeof(path) -1);
-	    if ( 0 < r ) {
-		path[r - 1] = '\0'; /* remove trailing \x0a */
-	    }
-	    close(fd);
-	}
-	gpsd_report(LOG_INF, "KPPS checking %s, %s\n"
-	    , globbuf.gl_pathv[i], path);
-	if ( 0 == strncmp( path, session->gpsdata.dev.path, sizeof(path))) {
-	    /* this is the pps we are looking for */
-	    /* FIXME, now build the proper pps device path */
-	    pps_num = globbuf.gl_pathv[i][28];
-	    break;
-	}
-	memset( (void *)&path, 0, sizeof(path));
-    }
-    /* done with blob, clear it */
-    globfree(&globbuf);
-
-    if ( 0 == pps_num ) {
-	gpsd_report(LOG_INF, "KPPS device not found.\n");
-    	return -1;
-    }
-    /* contruct the magic device path */
-    (void)snprintf(path, sizeof(path), "/dev/pps%c", pps_num);
-
-    int ret = open(path, O_RDWR);
-    if ( 0 > ret ) {
-	gpsd_report(LOG_INF, "KPPS cannot open %s: %d\n"
-	    , path, errno);
-    	return -1;
-    }
-
-    if ( 0 > time_pps_create(ret, &kernelpps_handle )) {
-	gpsd_report(LOG_INF, "KPPS time_pps_create(%d,) failed: %d\n"
-	    , ret, errno);
-    	return -1;
-    } else {
-    	/* have kernel PPS handle */
-        int caps;
-	/* get features  supported */
-        if ( 0 > time_pps_getcap(kernelpps_handle, &caps)) {
-	    gpsd_report(LOG_ERROR, "KPPS time_pps_getcap() failed\n");
-        } else {
-	    gpsd_report(LOG_INF, "KPPS caps %0x\n", caps);
-        }
-
-        /* linux 2.6.34 can not PPS_ECHOASSERT | PPS_ECHOCLEAR */
-        memset( (void *)&pp, 0, sizeof(pps_params_t));
-        pp.mode = PPS_CAPTUREBOTH;
-
-        if ( 0 > time_pps_setparams(kernelpps_handle, &pp)) {
-	    gpsd_report(LOG_ERROR, 
-	    	"KPPS time_pps_setparams() failed, errno:%d\n", errno);
-	    return -1;
-        }
-    }
-    return kernelpps_handle;
-}
-#endif
-
-/*@-mustfreefresh -type@ -unrecog*/
-static /*@null@*/ void *gpsd_ppsmonitor(void *arg)
-{
-    struct gps_device_t *session = (struct gps_device_t *)arg;
-    int cycle, duration, state = 0, laststate = -1, unchanged = 0;
-    struct timeval  tv;
-    struct timespec ts;
-    struct timeval pulse[2] = { {0, 0}, {0, 0} };
-#if defined(PPS_ON_CTS)
-    int pps_device = TIOCM_CTS;
-#define pps_device_str "CTS"
-#else
-    int pps_device = TIOCM_CAR;
-#define pps_device_str "DCD"
-#endif
-#if defined(HAVE_SYS_TIMEPPS_H)
-    int kpps_edge = 0;       /* 0 = clear edge, 1 = assert edge */
-    int cycle_kpps, duration_kpps;
-    struct timespec pulse_kpps[2] = { {0, 0}, {0, 0} };
-    struct timespec tv_kpps;
-    pps_info_t pi;
-#endif
-/* for chrony SOCK interface, which allows nSec timekeeping */
-#define SOCK_MAGIC 0x534f434b
-    struct sock_sample {
-	struct timeval tv;
-	double offset;
-	int pulse;
-	int leap;
-	int _pad;	/* unused */
-	int magic;      /* must be SOCK_MAGIC */
-    } sample;
-    /* chrony must be started first as chrony insists on creating the socket */
-    /* open the chrony socket */
-    struct sockaddr_un s;
-    int chronyfd;
-    char *chrony_path = NULL;
-
-    gpsd_report(LOG_PROG, "PPS Create Thread gpsd_ppsmonitor\n");
-#ifdef __UNUSED__
-    /* sadly root was dropped very early, until a way if found to run this
-     * before dropping root it will not work. */
-    if( 0 == getuid() ) {
-        /* only root can use /var/run */
-    	chrony_path = "/var/run/chrony";
-    } else
-#endif
-    {
-    	chrony_path = "/tmp/chrony";
-    }
-
-    s.sun_family = AF_UNIX;
-    (void)snprintf(s.sun_path, sizeof (s.sun_path), "%s.%s.sock", chrony_path,
-        basename(session->gpsdata.dev.path));
-    /* TODO the socket for root would be
-     *   /var/run/chrony.ttyXX.sock
-     * for now it is always
-     *   /tmp/chrony.ttyXX.sock
-     */
-
-    chronyfd = socket(AF_UNIX, SOCK_DGRAM, 0);
-    if (chronyfd < 0) {
-	gpsd_report(LOG_PROG, "PPS can not open chrony socket: %s\n",
-	    s.sun_path);
-    } else if (connect(chronyfd, (struct sockaddr *)&s, (int)sizeof(s))) {
-	(void)close(chronyfd);
-	chronyfd = -1;
-	gpsd_report(LOG_PROG, "PPS can not connect chrony socket: %s\n", 
-	    s.sun_path);
-    } else {
-	gpsd_report(LOG_RAW, "PPS using chrony socket: %s\n", s.sun_path);
-    }
-
-    /* end chrony */
-
-#if defined(HAVE_SYS_TIMEPPS_H)
-    int kernelpps_handle = init_kernel_pps( session );
-    if ( 0 <= kernelpps_handle ) {
-	gpsd_report(LOG_WARN, "KPPS kernel PPS will be used\n");
-    }
-    memset( (void *)&pi, 0, sizeof(pps_info_t));
-
-#endif
-
-    /* wait for status change on the device's carrier-detect line */
-    while (ioctl(session->gpsdata.gps_fd, TIOCMIWAIT, pps_device) == 0) {
-	int ok = 0;
-	char *log = NULL;
-
-#ifdef HAVE_LIBRT
-	/* using  clock_gettime() here, that is nSec, 
-	 * not uSec like gettimeofday */
-	if ( 0 > clock_gettime(CLOCK_REALTIME, &ts) ) {
-	    /* uh, oh, can not get time! */
-	    continue;
-	}
-	TSTOTV( &tv, &ts);
-#else
-	if ( 0 > gettimeofday(&tv, NULL) ) {
-	    /* uh, oh, can not get time! */
-	    continue;
-	}
-	TVTOTS( &ts, &tv);
-#endif
-
-#if defined(HAVE_SYS_TIMEPPS_H)
-        if ( 0 <= kernelpps_handle ) {
-	    struct timespec kernelpps_tv;
-	    /* on a quad core 2.4GHz Xeon this removes about 20uS of 
-	     * latency, and about +/-5uS of jitter over the other method */
-            memset( (void *)&kernelpps_tv, 0, sizeof(kernelpps_tv));
-	    if ( 0 > time_pps_fetch(kernelpps_handle, PPS_TSFMT_TSPEC
-	        , &pi, &kernelpps_tv)) {
-		gpsd_report(LOG_ERROR, "KPPS kernel PPS failed\n");
-	    } else {
-		// find the last edge
-	    	if ( pi.assert_timestamp.tv_sec > pi.clear_timestamp.tv_sec ) {
-		    kpps_edge = 1;
-		    tv_kpps = pi.assert_timestamp;
-	    	} else if ( pi.assert_timestamp.tv_sec < pi.clear_timestamp.tv_sec ) {
-		    kpps_edge = 0;
-		    tv_kpps = pi.clear_timestamp;
-		} else if ( pi.assert_timestamp.tv_nsec > pi.clear_timestamp.tv_nsec ) {
-		    kpps_edge = 1;
-		    tv_kpps = pi.assert_timestamp;
-		} else {
-		    kpps_edge = 0;
-		    tv_kpps = pi.clear_timestamp;
-		}
-		gpsd_report(LOG_PROG, "assert %ld.%09ld, sequence: %ld - "
-		       "clear  %ld.%09ld, sequence: %ld\n",
-		       pi.assert_timestamp.tv_sec,
-		       pi.assert_timestamp.tv_nsec,
-		       pi.assert_sequence,
-		       pi.clear_timestamp.tv_sec,
-		       pi.clear_timestamp.tv_nsec, 
-		       pi.clear_sequence);
-		gpsd_report(LOG_PROG, "KPPS data: using %s\n",
-		       kpps_edge ? "assert" : "clear");
-
-#define timediff_kpps(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+((x.tv_nsec-y.tv_nsec)/1000))
-	        cycle_kpps = timediff_kpps(tv_kpps, pulse_kpps[kpps_edge]);
-	        duration_kpps = timediff_kpps(tv_kpps, pulse_kpps[(int)(kpps_edge == 0)]);
-		if ( 3000000 < duration_kpps ) {
-		    // invisible pulse
-		    duration_kpps = 0;
-		}
-#undef timediff_kpps
-	        gpsd_report(LOG_INF, 
-		    "KPPS cycle: %7d, duration: %7d @ %lu.%09lu\n",
-		    cycle_kpps, duration_kpps,
-		    (unsigned long)tv_kpps.tv_sec, 
-		    (unsigned long)tv_kpps.tv_nsec);
-		pulse_kpps[kpps_edge] = tv_kpps;
-	    }
-	}
-#endif
-
-	ok = 0;
-	log = NULL;
-
-	/*@ +ignoresigns */
-	if (ioctl(session->gpsdata.gps_fd, TIOCMGET, &state) != 0)
-	    break;
-	/*@ -ignoresigns */
-
-	state = (int)((state & pps_device) != 0);
-	/*@ +boolint @*/
-#define timediff(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+x.tv_usec-y.tv_usec)
-	cycle = timediff(tv, pulse[state]);
-	duration = timediff(tv, pulse[(int)(state == 0)]);
-#undef timediff
-	/*@ -boolint @*/
-
-	if (state == laststate) {
-	    /* some pulses may be so short that state never changes */
-	    if (999000 < cycle && 1001000 > cycle) {
-		duration = 0;
-		unchanged = 0;
-		gpsd_report(LOG_RAW,
-			    "PPS pps-detect (%s) on %s invisible pulse\n",
-			    pps_device_str, session->gpsdata.dev.path);
-	    } else if (++unchanged == 10) {
-		unchanged = 1;
-		gpsd_report(LOG_WARN,
-			    "PPS TIOCMIWAIT returns unchanged state, ppsmonitor sleeps 10\n");
-		(void)sleep(10);
-	    }
-	} else {
-	    gpsd_report(LOG_RAW, "PPS pps-detect (%s) on %s changed to %d\n",
-			pps_device_str, session->gpsdata.dev.path, state);
-	    laststate = state;
-	    unchanged = 0;
-	}
-	pulse[state] = tv;
-	if (unchanged) {
-	    // strange, try again
-	    continue;
-	}
-	gpsd_report(LOG_INF, "PPS  cycle: %7d, duration: %7d @ %lu.%06lu\n",
-		    cycle, duration,
-		    (unsigned long)tv.tv_sec, (unsigned long)tv.tv_usec);
-
-	/*@ +boolint @*/
-	if (3 < session->context->fixcnt
-#if defined(ONCORE_ENABLE) && defined(BINARY_ENABLE)
-	    || (session->device_type == &oncore_binary &&
-		session->driver.oncore.good_time)
-#endif
-	    ) {
-	    /* Garmin doc says PPS is valid after four good fixes. */
-	    /*
-	     * The PPS pulse is normally a short pulse with a frequency of
-	     * 1 Hz, and the UTC second is defined by the front edge. But we
-	     * don't know the polarity of the pulse (different receivers
-	     * emit different polarities). The duration variable is used to
-	     * determine which way the pulse is going. The code assumes
-	     * that the UTC second is changing when the signal has not
-	     * been changing for at least 800ms, i.e. it assumes the duty
-	     * cycle is at most 20%.
-	     *
-	     * Some GPS instead output a square wave that is 0.5 Hz and each
-	     * edge denotes the start of a second.
-	     *
-	     * Some GPS, like the Globalsat MR-350P, output a 1uS pulse.
-	     * The pulse is so short that TIOCMIWAIT sees a state change
-	     * but by the time TIOCMGET is called the pulse is gone.
-	     *
-	     * A few stupid GPS, like the Furuno GPSClock, output a 1.0 Hz
-	     * square wave where the leading edge is the start of a second
-	     *
-	     * 5Hz GPS (Garmin 18-5Hz) pulses at 5Hz. Set the pulse length to
-	     * 40ms which gives a 160ms pulse before going high.
-	     *
-	     */
-
-	    if (199000 > cycle) {
-		// too short to even be a 5Hz pulse
-		log = "Too short for 5Hz\n";
-	    } else if (201000 > cycle) {
-		/* 5Hz cycle */
-		/* looks like 5hz PPS pulse */
-		if (100000 > duration) {
-		    /* BUG: how does the code know to tell ntpd
-		     * which 1/5 of a second to use?? */
-		    ok = 1;
-		    log = "5Hz PPS pulse\n";
-		}
-	    } else if (999000 > cycle) {
-		log = "Too long for 5Hz, too short for 1Hz\n";
-	    } else if (1001000 > cycle) {
-		/* looks like PPS pulse or square wave */
-		if (0 == duration) {
-		    ok = 1;
-		    log = "PPS invisible pulse\n";
-		} else if (499000 > duration) {
-		    /* end of the short "half" of the cycle */
-		    /* aka the trailing edge */
-		    log = "PPS 1Hz trailing edge\n";
-		} else if (501000 > duration) {
-		    /* looks like 1.0 Hz square wave, ignore trailing edge */
-		    if (state == 1) {
-			ok = 1;
-			log = "PPS square\n";
-		    }
-		} else {
-		    /* end of the long "half" of the cycle */
-		    /* aka the leading edge */
-		    ok = 1;
-		    log = "PPS 1Hz leading edge\n";
-		}
-	    } else if (1999000 > cycle) {
-		log = "Too long for 1Hz, too short for 2Hz\n";
-	    } else if (2001000 > cycle) {
-		/* looks like 0.5 Hz square wave */
-		if (999000 > duration) {
-		    log = "PPS 0.5 Hz square too short duration\n";
-		} else if (1001000 > duration) {
-		    ok = 1;
-		    log = "PPS 0.5 Hz square wave\n";
-		} else {
-		    log = "PPS 0.5 Hz square too long duration\n";
-		}
-	    } else {
-		log = "Too long for 0.5Hz\n";
-	    }
-	} else {
-	    /* not a good fix, but a test for an otherwise good PPS
-	     * would go here */
-	    log = "PPS no fix.\n";
-	}
-	/*@ -boolint @*/
-	if (NULL != log) {
-	    gpsd_report(LOG_RAW, "%.100s", log);
-	}
-	if (0 != ok) {
-	    /* chrony expects tv-sec since Jan 1970 */
-	    /* FIXME!! offset is double of the error from local time */
-	    sample.pulse = 0;
-	    sample.leap = 0;
-	    sample.magic = SOCK_MAGIC;
-#if defined(HAVE_SYS_TIMEPPS_H)
-            if ( 0 <= kernelpps_handle ) {
-		/* pick the right edge */
-		if ( kpps_edge ) {
-		    ts = pi.assert_timestamp; /* structure copy */
-		} else {
-		    ts = pi.clear_timestamp;  /* structure copy */
-		}
-		TSTOTV( &sample.tv, &ts);
-	    } else
-#endif
-	    {
-		sample.tv = tv; 	/* structure copy */
-	    } 
-	    /* FIXME!! this is wrong if signal is 5Hz or 10Hz instead of PPS */
-	    /* carefull, unix time to nSec is more precision that a double */
-	    sample.offset = 1 + session->last_fixtime - ts.tv_sec;
-	    sample.offset -= ts.tv_nsec / 1e9;
-#if defined(ONCORE_ENABLE) && defined(BINARY_ENABLE)
-	    /*@-noeffect@*/
-	    if (session->device_type == &oncore_binary) {
-		int pulse_delay_ns = session->driver.oncore.pps_offset_ns;
-	        sample.offset += (double)pulse_delay_ns / 1000000000;
-	        ts.tv_nsec    -= pulse_delay_ns;
-	        TS_NORM( &ts );
-	    }
-	    /*@+noeffect@*/
-#endif
-
-	    if ( 0 <= chronyfd ) {
-		(void)send(chronyfd, &sample, sizeof (sample), 0);
-		gpsd_report(LOG_RAW, "PPS chrony sock %lu.%06lu offset %.9f\n",
-			    (unsigned long)sample.tv.tv_sec,
-			    (unsigned long)sample.tv.tv_usec,
-			    sample.offset);
-	    }
-	    TSTOTV( &tv, &ts );
-	    (void)ntpshm_pps(session, &tv);
-	} else {
-	    gpsd_report(LOG_INF, "PPS edge rejected\n");
-	}
-
-    }
-
-    return NULL;
-}
-/*@+mustfreefresh +type +unrecog@*/
-#endif /* PPS_ENABLE */
-
 void gpsd_clear(struct gps_device_t *session)
 {
     session->gpsdata.online = timestamp();
@@ -712,6 +181,7 @@ void gpsd_clear(struct gps_device_t *session)
     session->driver.sirf.satcounter = 0;
 #endif /* SIRF_ENABLE */
     packet_init(&session->packet);
+    session->packet.debug = session->context->debug;
     // session->gpsdata.online = 0;
     session->gpsdata.fix.mode = MODE_NOT_SEEN;
     session->gpsdata.status = STATUS_NO_FIX;
@@ -730,6 +200,7 @@ void gpsd_clear(struct gps_device_t *session)
 int gpsd_open(struct gps_device_t *session)
 /* open a device for access to its data */
 {
+#ifdef NETFEED_ENABLE
     /* special case: source may be a URI to a remote GNSS or DGPS service */
     if (netgnss_uri_check(session->gpsdata.dev.path)) {
 	session->gpsdata.gps_fd = netgnss_uri_open(session,
@@ -786,6 +257,33 @@ int gpsd_open(struct gps_device_t *session)
 	session->sourcetype = source_udp;
 	return session->gpsdata.gps_fd;
     }
+#endif /* NETFEED_ENABLE */
+#ifdef PASSTHROUGH_ENABLE
+    else if (strncmp(session->gpsdata.dev.path, "gpsd://", 7) == 0) {
+	/*@-branchstate -nullpass@*/
+	char server[GPS_PATH_MAX], *port;
+	socket_t dsock;
+	(void)strlcpy(server, session->gpsdata.dev.path + 7, sizeof(server));
+	session->gpsdata.gps_fd = -1;
+	if ((port = strchr(server, ':')) == NULL) {
+	    port = DEFAULT_GPSD_PORT;
+	} else
+	    *port++ = '\0';
+	gpsd_report(LOG_INF, "opening remote gpsd feed at %s, port %s.\n", server,
+		    port);
+	if ((dsock = netlib_connectsock(AF_UNSPEC, server, port, "tcp")) < 0) {
+	    gpsd_report(LOG_ERROR, "remote gpsd device open error %s.\n",
+			netlib_errstr(dsock));
+	    return -1;
+	} else
+	    gpsd_report(LOG_SPIN, "remote gpsd feed opened on fd %d\n", dsock);
+	/*@+branchstate +nullpass@*/
+	/* watch to remote is issued when WATCH is */
+	session->gpsdata.gps_fd = dsock;
+	session->sourcetype = source_gpsd;
+	return session->gpsdata.gps_fd;
+    }
+#endif /* PASSTHROUGH_ENABLE */
 
     /* fall through to plain serial open */
     return gpsd_serial_open(session);
@@ -842,53 +340,7 @@ int gpsd_activate(struct gps_device_t *session)
 
 /*@ +branchstate @*/
 
-void ntpd_link_activate(struct gps_device_t *session)
-{
-#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-    pthread_t pt;
-#endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
-
-#ifdef NTPSHM_ENABLE
-    /* If we are talking to ntpd, allocate a shared-memory segment for "NMEA" time data */
-    if (session->context->enable_ntpshm)
-	session->shmindex = ntpshm_alloc(session->context);
-
-    if (0 > session->shmindex) {
-	gpsd_report(LOG_INF, "NTPD ntpshm_alloc() failed\n");
-#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-    } else if (session->context->shmTimePPS) {
-	/* We also have the 1pps capability, allocate a shared-memory segment
-	 * for the 1pps time data and launch a thread to capture the 1pps
-	 * transitions
-	 */
-	if ((session->shmTimeP = ntpshm_alloc(session->context)) >= 0) {
-	    /*@-unrecog@*/
-	    (void)pthread_create(&pt, NULL, gpsd_ppsmonitor, (void *)session);
-	    /*@+unrecog@*/
-	} else {
-	    gpsd_report(LOG_INF, "NTPD ntpshm_alloc(1) failed\n");
-	}
-
-#endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
-    }
-#endif /* NTPSHM_ENABLE */
-}
-
-char /*@observer@*/ *gpsd_id( /*@in@ */ struct gps_device_t *session)
-/* full ID of the device for reports, including subtype */
-{
-    static char buf[128];
-    if ((session == NULL) || (session->device_type == NULL) ||
-	(session->device_type->type_name == NULL))
-	return "unknown,";
-    (void)strlcpy(buf, session->device_type->type_name, sizeof(buf));
-    if (session->subtype[0] != '\0') {
-	(void)strlcat(buf, " ", sizeof(buf));
-	(void)strlcat(buf, session->subtype, sizeof(buf));
-    }
-    return (buf);
-}
-
+#ifdef CHEAPFLOATS_ENABLE
 /*****************************************************************************
 
 Carl Carter of SiRF supplied this algorithm for computing DOPs from
@@ -977,12 +429,6 @@ driver.
 
 ******************************************************************************/
 
-void clear_dop( /*@out@*/ struct dop_t *dop)
-{
-    dop->xdop = dop->ydop = dop->vdop = dop->tdop = dop->hdop = dop->pdop =
-	dop->gdop = NAN;
-}
-
 /*@ -fixedformalarray -mustdefine @*/
 static bool invert(double mat[4][4], /*@out@*/ double inverse[4][4])
 {
@@ -1111,7 +557,7 @@ static gps_mask_t fill_dop(const struct gps_data_t * gpsdata, struct dop_t * dop
     if (n < 4) {
 	gpsd_report(LOG_DATA + 2, "Not enough Satellites available %d < 4:\n",
 		    n);
-	return 0;		/* Is this correct return code here? or should it be ERROR_IS */
+	return 0;		/* Is this correct return code here? or should it be ERROR_SET */
     }
 
     memset(prod, 0, sizeof(prod));
@@ -1202,7 +648,7 @@ static gps_mask_t fill_dop(const struct gps_data_t * gpsdata, struct dop_t * dop
     }
     /*@ +usedef @*/
 
-    return DOP_IS;
+    return DOP_SET;
 }
 
 static void gpsd_error_model(struct gps_device_t *session,
@@ -1276,8 +722,6 @@ static void gpsd_error_model(struct gps_device_t *session,
     if (isnan(fix->time) == 0 && isnan(fix->ept) != 0)
 	fix->ept = 0.005;
     /* Other error computations depend on having a valid fix */
-    gpsd_report(LOG_DATA, "modeling errors: mode=%d, masks=%s\n",
-		fix->mode, gpsd_maskdump(session->gpsdata.set));
     if (fix->mode >= MODE_2D) {
 	if (isnan(fix->epx) != 0 && isfinite(session->gpsdata.dop.hdop) != 0)
 	    fix->epx = session->gpsdata.dop.xdop * h_uere;
@@ -1356,6 +800,7 @@ static void gpsd_error_model(struct gps_device_t *session,
 	(void)memcpy(oldfix, fix, sizeof(struct gps_fix_t));
     /*@ +mayaliasunique @*/
 }
+#endif /* CHEAPFLOATS_ENABLE */
 
 gps_mask_t gpsd_poll(struct gps_device_t *session)
 /* update the stuff in the scoreboard structure */
@@ -1384,25 +829,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		    session->gpsdata.dev.path,
 		    session->device_type->type_name);
     } else {
-	const struct gps_type_t **dp;
-
 	newlen = generic_get(session);
-	gpsd_report(LOG_RAW,
-		    "packet sniff on %s finds type %d\n",
-		    session->gpsdata.dev.path, session->packet.type);
-	if (session->packet.type == COMMENT_PACKET) {
-	    gpsd_report (LOG_PROG, "comment, sync lock deferred\n");
-	} else if (session->packet.type > COMMENT_PACKET) {
-	    first_sync = (session->device_type == NULL);
-	    for (dp = gpsd_drivers; *dp; dp++)
-		if (session->packet.type == (*dp)->packet_type) {
-		    (void)gpsd_switch_driver(session, (*dp)->type_name);
-		    break;
-		}
-	} else if (session->getcount++ > 1 && !gpsd_next_hunt_setting(session)) {
-	    gpsd_run_device_hook(session->gpsdata.dev.path, "DEACTIVATE");
-	    return ERROR_IS;
-	}
     }
 
     /* update the scoreboard structure from the GPS */
@@ -1413,7 +840,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		    session->gpsdata.dev.path, newlen,
 		    timestamp() - session->gpsdata.online);
 	session->gpsdata.online = (timestamp_t)0;
-	return ERROR_IS;
+	return ERROR_SET;
     } else if (newlen == 0) {		/* zero length read, possible EOF */
 	/*
 	 * Multiplier is 2 to avoid edge effects due to sampling at the exact
@@ -1426,12 +853,38 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    session->gpsdata.online = (timestamp_t)0;
 	}
 	return NODATA_IS;
-    } else if (session->packet.outbuflen == 0) {	/* got new data, but no packet */
+    } else /* (newlen > 0) */ {
+	const struct gps_type_t **dp;
+
+	gpsd_report(LOG_RAW,
+		    "packet sniff on %s finds type %d\n",
+		    session->gpsdata.dev.path, session->packet.type);
+	if (session->packet.type == COMMENT_PACKET) {
+	    gpsd_report (LOG_PROG, "comment, sync lock deferred\n");
+	    /* FALL THROUGH */
+	} else if (session->packet.type > COMMENT_PACKET) {
+	    first_sync = (session->device_type == NULL);
+	    for (dp = gpsd_drivers; *dp; dp++)
+		if (session->packet.type == (*dp)->packet_type) {
+		    (void)gpsd_switch_driver(session, (*dp)->type_name);
+		    break;
+		}
+	    /* FALL THROUGH */
+	} else if (session->getcount++>1 && !gpsd_next_hunt_setting(session)) {
+	    gpsd_run_device_hook(session->gpsdata.dev.path, "DEACTIVATE");
+	    gpsd_report(LOG_INF, "hunt on %s failed (%lf sec since data)\n",
+			session->gpsdata.dev.path,
+			timestamp() - session->gpsdata.online);
+	    return ERROR_SET;
+	}
+    }
+
+    if (session->packet.outbuflen == 0) {	/* got new data, but no packet */
 	gpsd_report(LOG_RAW + 3, "New data on %s, not yet a packet\n",
 		    session->gpsdata.dev.path);
-	return ONLINE_IS;
+	return ONLINE_SET;
     } else {			/* we have recognized a packet */
-	gps_mask_t received = PACKET_IS, dopmask = 0;
+	gps_mask_t received = PACKET_SET;
 	session->gpsdata.online = timestamp();
 
 	gpsd_report(LOG_RAW + 3, "Accepted packet on %s.\n",
@@ -1477,57 +930,37 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    received |= DRIVER_IS;
 	}
 
+	/*
+	 * The guard looks superfluous, but it keeps the rather expensive
+	 * gpsd_hexdump() function from being called even when the debug
+	 * level does not actually require it.
+	 */
+	if (session->context->debug >= LOG_RAW)
+	    gpsd_report(LOG_RAW, "raw packet of type %d, %zd:%s\n",
+			session->packet.type,
+			session->packet.outbuflen,
+			gpsd_hexdump((char *)session->packet.outbuffer, session->packet.outbuflen));
+
 	/* Get data from current packet into the fix structure */
 	if (session->packet.type != COMMENT_PACKET)
 	    if (session->device_type != NULL
 		&& session->device_type->parse_packet != NULL)
 		received |= session->device_type->parse_packet(session);
 
-#ifdef NTPSHM_ENABLE
-	/*
-	 * Only update the NTP time if we've seen the leap-seconds data.
-	 * Else we may be providing GPS time.
-	 */
-	if (session->context->enable_ntpshm == 0) {
-	    //gpsd_report(LOG_PROG, "NTP: off\n");
-	} else if ((received & TIME_IS) == 0) {
-	    //gpsd_report(LOG_PROG, "NTP: No time this packet\n");
-	} else if (isnan(session->newdata.time)) {
-	    //gpsd_report(LOG_PROG, "NTP: bad new time\n");
-	} else if (session->newdata.time == session->last_fixtime) {
-	    //gpsd_report(LOG_PROG, "NTP: Not a new time\n");
-	} else if (session->newdata.mode == MODE_NO_FIX
-#if defined(ONCORE_ENABLE) && defined(BINARY_ENABLE)
-		   && !(session->device_type == &oncore_binary &&
-			session->driver.oncore.good_time!=0)
-#endif
-		   ) {
-	    //gpsd_report(LOG_PROG, "NTP: No fix\n");
-	} else {
-	    double offset;
-	    //gpsd_report(LOG_PROG, "NTP: Got one\n");
-	    /* assume zero when there's no offset method */
-	    if (session->device_type == NULL
-		|| session->device_type->ntp_offset == NULL)
-		offset = 0.0;
-	    else
-		offset = session->device_type->ntp_offset(session);
-	    (void)ntpshm_put(session, session->newdata.time, offset);
-	    session->last_fixtime = session->newdata.time;
-	}
-#endif /* NTPSHM_ENABLE */
+	session->gpsdata.set = ONLINE_SET | received;
 
+#ifdef CHEAPFLOATS_ENABLE
 	/*
 	 * Compute fix-quality data from the satellite positions.
 	 * These will not overwrite any DOPs reported from the packet
 	 * we just got.
 	 */
-	if ((received & SATELLITE_IS) != 0
+	if ((received & SATELLITE_SET) != 0
 	    && session->gpsdata.satellites_visible > 0) {
-	    dopmask = fill_dop(&session->gpsdata, &session->gpsdata.dop);
+	    session->gpsdata.set |= fill_dop(&session->gpsdata, &session->gpsdata.dop);
 	    session->gpsdata.epe = NAN;
 	}
-	session->gpsdata.set = ONLINE_IS | dopmask | received;
+#endif /* CHEAPFLOATS_ENABLE */
 
 	/* copy/merge device data into staging buffers */
 	/*@-nullderef -nullpass@*/
@@ -1535,12 +968,15 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    gps_clear_fix(&session->gpsdata.fix);
 	/* don't downgrade mode if holding previous fix */
 	if (session->gpsdata.fix.mode > session->newdata.mode)
-	    session->gpsdata.set &= ~MODE_IS;
+	    session->gpsdata.set &= ~MODE_SET;
 	//gpsd_report(LOG_PROG,
 	//              "transfer mask on %s: %02x\n", session->gpsdata.tag, session->gpsdata.set);
 	gps_merge_fix(&session->gpsdata.fix,
 		      session->gpsdata.set, &session->newdata);
+#ifdef CHEAPFLOATS_ENABLE
 	gpsd_error_model(session, &session->gpsdata.fix, &session->oldfix);
+#endif /* CHEAPFLOATS_ENABLE */
+
 	/*@+nullderef -nullpass@*/
 
 	/*
@@ -1555,9 +991,11 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	 * devices output fix packets on a regular basis, even when unable
 	 * to derive a good fix. Such packets should set STATUS_NO_FIX.
 	 */
-	if ((session->gpsdata.set & LATLON_IS) != 0
-	    && session->gpsdata.status > STATUS_NO_FIX)
+	if ((session->gpsdata.set & LATLON_SET) != 0
+	    && session->gpsdata.status > STATUS_NO_FIX) {
 	    session->context->fixcnt++;
+	    session->fixcnt++;
+	}
 
 #ifdef TIMING_ENABLE
 	session->d_decode_time = timestamp();
@@ -1568,12 +1006,12 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	 * driver errors, including 32-vs.-64-bit problems.
 	 */
 	/*@+relaxtypes +longunsignedintegral@*/
-	if ((session->gpsdata.set & TIME_IS) != 0) {
+	if ((session->gpsdata.set & TIME_SET) != 0) {
 	    if (session->newdata.time > time(NULL) + (60 * 60 * 24 * 365))
 		gpsd_report(LOG_WARN,
 			    "date more than a year in the future!\n");
 	    else if (session->newdata.time < 0)
-		gpsd_report(LOG_ERROR, "date is negative!\n");
+		gpsd_report(LOG_ERROR, "date in %s is negative!\n", session->gpsdata.tag);
 	}
 	/*@-relaxtypes -longunsignedintegral@*/
 
@@ -1595,5 +1033,5 @@ void gpsd_zero_satellites( /*@out@*/ struct gps_data_t *out)
     (void)memset(out->azimuth, 0, sizeof(out->azimuth));
     (void)memset(out->ss, 0, sizeof(out->ss));
     out->satellites_visible = 0;
-    clear_dop(&out->dop);
+    gps_clear_dop(&out->dop);
 }
