@@ -197,12 +197,12 @@ static bool sirf_initialize(void)
     /*@ +nullpass @*/
     /*@ -onlytrans @*/
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
     /* probe for version */
     /*@ -compdef @*/
     (void)monitor_control_send((unsigned char *)"\x84\x00", 2);
     /*@ +compdef @*/
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
     /* initialize the GPS context's time fields */
     gpsd_time_init(session.context, time(NULL));
@@ -271,7 +271,7 @@ static void sirf_update(void)
     unsigned char *buf;
     size_t len;
     uint8_t dgps;
-    char tbuf[128];
+    char tbuf[JSON_DATE_MAX+1];
 
     assert(mid27win != NULL);
     buf = session.packet.outbuffer + 4;
@@ -292,7 +292,7 @@ static void sirf_update(void)
 								  17) / 8);
 	/* line 3 */
 	(void)wmove(mid2win, 3, 7);
-	(void)wprintw(mid2win, "%-22s",
+	(void)wprintw(mid2win, "%-24s",
 			unix_to_iso8601(session.gpsdata.fix.time, tbuf, sizeof(tbuf))
 			);
 	(void)wmove(mid2win, 3, 38);
@@ -631,12 +631,12 @@ static void sirf_update(void)
 	break;
     }
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
     /* elicit navigation parameters */
     if (dispmode && (time(NULL) % 10 == 0)) {
 	(void)monitor_control_send((unsigned char *)"\x98\x00", 2);
     }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
     /*@ -nullpass -nullderef @*/
     if (dispmode) {
@@ -648,7 +648,7 @@ static void sirf_update(void)
 
 /*@ +globstate */
 
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
 static int sirf_command(char line[])
 {
     unsigned char buf[BUFSIZ];
@@ -714,7 +714,7 @@ static int sirf_command(char line[])
 
     return COMMAND_UNKNOWN;	/* no match */
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 
 static void sirf_wrap(void)
 {
@@ -731,11 +731,11 @@ static void sirf_wrap(void)
 const struct monitor_object_t sirf_mmt = {
     .initialize = sirf_initialize,
     .update = sirf_update,
-#ifdef ALLOW_CONTROLSEND
+#ifdef CONTROLSEND_ENABLE
     .command = sirf_command,
 #else
     .command = NULL,
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
     .wrap = sirf_wrap,
     .min_y = 22,.min_x = 80,
     .driver = &sirf_binary,
