@@ -465,15 +465,17 @@ int gpsd_serial_open(struct gps_device_t *session)
 	session->ttyset.c_iflag = session->ttyset.c_oflag =
 	    session->ttyset.c_lflag = (tcflag_t) 0;
 
+#ifndef FIXED_PORT_SPEED
 	session->baudindex = 0;
+#endif /* FIXED_PORT_SPEED */
 	gpsd_set_speed(session, gpsd_get_speed(&session->ttyset_old), 'N', 1);
     }
-    gpsd_report(LOG_SPIN, "open(%s) -> %d in gpsd_open()\n",
+    gpsd_report(LOG_SPIN, "open(%s) -> %d in gpsd_serial_open()\n",
 		session->gpsdata.dev.path, session->gpsdata.gps_fd);
     return session->gpsdata.gps_fd;
 }
 
-ssize_t gpsd_write(struct gps_device_t * session, char const *buf, size_t len)
+ssize_t gpsd_write(struct gps_device_t * session, const char *buf, size_t len)
 {
     ssize_t status;
     bool ok;
@@ -483,9 +485,10 @@ ssize_t gpsd_write(struct gps_device_t * session, char const *buf, size_t len)
     status = write(session->gpsdata.gps_fd, buf, len);
     ok = (status == (ssize_t) len);
     (void)tcdrain(session->gpsdata.gps_fd);
-    /* no test here now, always print as hex */
-    gpsd_report(LOG_IO, "=> GPS: %s%s\n",
-		gpsd_hexdump_wrapper(buf, len, LOG_IO), ok ? "" : " FAILED");
+    /* extra guard prevents expensive hexdump calls */
+    if (session->context->debug >= LOG_IO)
+	gpsd_report(LOG_IO, "=> GPS: %s%s\n",
+		    gpsd_hexdump((char *)buf, len), ok ? "" : " FAILED");
     return status;
 }
 
@@ -500,10 +503,7 @@ ssize_t gpsd_write(struct gps_device_t * session, char const *buf, size_t len)
 bool gpsd_next_hunt_setting(struct gps_device_t * session)
 /* advance to the next hunt setting  */
 {
-#ifdef FIXED_PORT_SPEED
-    /* just the one fixed port speed... */
-    static unsigned int rates[] = { FIXED_PORT_SPEED };
-#else /* FIXED_PORT_SPEED not defined */
+#ifndef FIXED_PORT_SPEED
     /* every rate we're likely to see on a GPS */
     static unsigned int rates[] =
 	{ 0, 4800, 9600, 19200, 38400, 57600, 115200 };
@@ -515,16 +515,33 @@ bool gpsd_next_hunt_setting(struct gps_device_t * session)
 
     if (session->packet.retry_counter++ >= SNIFF_RETRIES) {
 	session->packet.retry_counter = 0;
+#ifdef FIXED_PORT_SPEED
+	return false;
+#else
 	if (session->baudindex++ >=
 	    (unsigned int)(sizeof(rates) / sizeof(rates[0])) - 1) {
 	    session->baudindex = 0;
+#ifdef FIXED_STOP_BITS
+	    return false;	/* hunt is over, no sync */
+#else
 	    if (session->gpsdata.dev.stopbits++ >= 2)
 		return false;	/* hunt is over, no sync */
+#endif /* FIXED_STOP_BITS */
 	}
+#endif /* FIXED_PORT_SPEED */
 	gpsd_set_speed(session,
+#ifdef FIXED_PORT_SPEED
+		       FIXED_PORT_SPEED,
+#else
 		       rates[session->baudindex],
+#endif /* FIXED_PORT_SPEED */
 		       session->gpsdata.dev.parity,
-		       session->gpsdata.dev.stopbits);
+#ifdef FIXED_STOP_BITS
+		       FIXED_STOP_BITS,
+#else
+		       session->gpsdata.dev.stopbits
+#endif /* FIXED_STOP_BITS */
+	    );
     }
 
     return true;		/* keep hunting */
@@ -541,22 +558,6 @@ void gpsd_assert_sync(struct gps_device_t *session)
      */
     if (session->saved_baud == -1)
 	session->saved_baud = (int)cfgetispeed(&session->ttyset);
-
-#ifdef NTPSHM_ENABLE
-    /*
-     * Now is the right time to grab the shared memory segment(s)
-     * to communicate the navigation message derived and (possibly)
-     * 1pps derived time data to ntpd.
-     */
-
-    /* do not start more than one ntp thread */
-    if (!(session->shmindex >= 0))
-	ntpd_link_activate(session);
-
-    gpsd_report(LOG_INF, "NTPD ntpd_link_activate: %d\n",
-		(int)session->shmindex >= 0);
-
-#endif /* NTPSHM_ENABLE */
 }
 
 void gpsd_close(struct gps_device_t *session)
